# -*- coding: utf-8 -*-


import datetime
import json

from .common import *
from oss2 import to_string


class TestBucket(OssTestCase):
    def test_bucket(self):
        auth = oss2.Auth(OSS_ID, OSS_SECRET)
        bucket_name = OSS_BUCKET + "-test-bucket"
        bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)

        bucket.create_bucket(oss2.BUCKET_ACL_PRIVATE)

        service = oss2.Service(auth, OSS_ENDPOINT)
        wait_meta_sync()
        self.retry_assert(lambda: bucket.bucket_name in
                          (b.name for b in
                           service.list_buckets(prefix=bucket.bucket_name).buckets))

        key = 'a.txt'
        bucket.put_object(key, 'content')

        self.assertRaises(oss2.exceptions.BucketNotEmpty, bucket.delete_bucket)

        bucket.delete_object(key)
        bucket.delete_bucket()

        wait_meta_sync()
        self.assertRaises(oss2.exceptions.NoSuchBucket, bucket.delete_bucket)

    def test_bucket_with_storage_class(self):
        auth = oss2.Auth(OSS_ID, OSS_SECRET)
        bucket_name = OSS_BUCKET + "-test-storage-class"
        bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)

        bucket.create_bucket(oss2.BUCKET_ACL_PRIVATE, oss2.models.BucketCreateConfig(oss2.BUCKET_STORAGE_CLASS_IA))

        service = oss2.Service(auth, OSS_ENDPOINT)
        wait_meta_sync()
        self.retry_assert(lambda: bucket.bucket_name in
                          (b.name for b in
                           service.list_buckets(prefix=bucket.bucket_name).buckets))

        key = 'a.txt'
        bucket.put_object(key, 'content')

        self.assertRaises(oss2.exceptions.BucketNotEmpty, bucket.delete_bucket)

        objects = bucket.list_objects()
        self.assertEqual(1, len(objects.object_list))
        self.assertEqual(objects.object_list[0].storage_class, 'IA')

        bucket.delete_object(key)
        bucket.delete_bucket()

        wait_meta_sync()
        self.assertRaises(oss2.exceptions.NoSuchBucket, bucket.delete_bucket)

    def test_bucket_with_data_redundancy_type(self):
        auth = oss2.Auth(OSS_ID, OSS_SECRET)
        bucket_name = OSS_BUCKET + "-test-redundancy-type"
        bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)

        # LRS
        bucketConfig = oss2.models.BucketCreateConfig(oss2.BUCKET_STORAGE_CLASS_IA, oss2.BUCKET_DATA_REDUNDANCY_TYPE_LRS)
        bucket.create_bucket(oss2.BUCKET_ACL_PRIVATE, bucketConfig)
        wait_meta_sync()
        
        result = bucket.get_bucket_info()
        self.assertEqual(oss2.BUCKET_DATA_REDUNDANCY_TYPE_LRS, result.data_redundancy_type)
        bucket.delete_bucket()

        # ZRS
        bucketConfig = oss2.models.BucketCreateConfig(oss2.BUCKET_STORAGE_CLASS_IA, oss2.BUCKET_DATA_REDUNDANCY_TYPE_ZRS)
        bucket.create_bucket(oss2.BUCKET_ACL_PRIVATE, bucketConfig)
        wait_meta_sync()
        
        result = bucket.get_bucket_info()
        self.assertEqual(oss2.BUCKET_DATA_REDUNDANCY_TYPE_ZRS, result.data_redundancy_type)
        bucket.delete_bucket()

    def test_acl(self):
        auth = oss2.Auth(OSS_ID, OSS_SECRET)
        bucket_name = OSS_BUCKET + "-test-acl"
        bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)

        bucket.create_bucket(oss2.BUCKET_ACL_PUBLIC_READ)

        self.retry_assert(lambda: bucket.get_bucket_acl().acl == oss2.BUCKET_ACL_PUBLIC_READ)

        bucket.put_bucket_acl(oss2.BUCKET_ACL_PRIVATE)
        wait_meta_sync()
        self.retry_assert(lambda: bucket.get_bucket_acl().acl == oss2.BUCKET_ACL_PRIVATE)

        bucket.put_bucket_acl(oss2.BUCKET_ACL_PUBLIC_READ_WRITE)
        wait_meta_sync()
        self.retry_assert(lambda: bucket.get_bucket_acl().acl == oss2.BUCKET_ACL_PUBLIC_READ_WRITE)

        bucket.delete_bucket()

    def test_logging(self):
        bucket_name = OSS_BUCKET + "-test-logging"
        other_bucket = oss2.Bucket(self.bucket.auth, OSS_ENDPOINT, bucket_name)
        other_bucket.create_bucket(oss2.BUCKET_ACL_PRIVATE)

        def same_logging(bucket_logging, target_bucket, target_prefix):
            return bucket_logging.target_bucket == target_bucket and bucket_logging.target_prefix == target_prefix

        for prefix in [u'日志+/', 'logging/', '日志+/']:
            other_bucket.put_bucket_logging(oss2.models.BucketLogging(self.bucket.bucket_name, prefix))
            wait_meta_sync()

            self.retry_assert(lambda: same_logging(other_bucket.get_bucket_logging(),
                                                   self.bucket.bucket_name,
                                                   to_string(prefix)))

        other_bucket.delete_bucket_logging()
        other_bucket.delete_bucket_logging()

        self.retry_assert(lambda: same_logging(other_bucket.get_bucket_logging(), '', ''))

        other_bucket.delete_bucket()

    @staticmethod
    def same_lifecycle(orig_rule, bucket):
        try:
            rules = bucket.get_bucket_lifecycle().rules
        except oss2.exceptions.NoSuchLifecycle:
            return False

        if not rules:
            return False

        rule_got = rules[0]
        if orig_rule.id != rule_got.id:
            return False

        if to_string(orig_rule.prefix) != rule_got.prefix:
            return False

        if orig_rule.status != rule_got.status:
            return False

        if orig_rule.expiration.days != rule_got.expiration.days:
            return False

        if orig_rule.expiration.date != rule_got.expiration.date:
            return False

        return True

    def test_lifecycle_days(self):
        from oss2.models import LifecycleExpiration, LifecycleRule, BucketLifecycle

        for prefix in ['中文前缀+/', '', u'中文前缀+/']:
            rule = LifecycleRule(random_string(10), prefix,
                                 status=LifecycleRule.ENABLED,
                                 expiration=LifecycleExpiration(days=356))
            lifecycle = BucketLifecycle([rule])

            self.bucket.put_bucket_lifecycle(lifecycle)
            self.retry_assert(lambda: self.same_lifecycle(rule, self.bucket))

        self.bucket.delete_bucket_lifecycle()
        self.bucket.delete_bucket_lifecycle()

        self.assertRaises(oss2.exceptions.NoSuchLifecycle, self.bucket.get_bucket_lifecycle)

    def test_put_lifecycle_days_less_than_transition_days(self):
        from oss2.models import LifecycleExpiration, LifecycleRule, BucketLifecycle

        rule = LifecycleRule(random_string(10), '中文前缀/',
                             status=LifecycleRule.ENABLED,
                             expiration=LifecycleExpiration(days=3))

        rule.storage_transitions = [oss2.models.StorageTransition(days=4, storage_class=oss2.BUCKET_STORAGE_CLASS_IA)]
        self.assertRaises(oss2.exceptions.InvalidArgument, self.bucket.put_bucket_lifecycle, BucketLifecycle([rule]))

    def test_put_lifecycle_invalid_transitions(self):
        from oss2.models import LifecycleExpiration, LifecycleRule, BucketLifecycle

        rule = LifecycleRule(random_string(10), '中文前缀/',
                             status=LifecycleRule.ENABLED,
                             expiration=LifecycleExpiration(days=6))
        # 转储为ARCHIVE的天数小于转储为IA
        rule.storage_transitions = [oss2.models.StorageTransition(days=5,
                                                                  storage_class=oss2.BUCKET_STORAGE_CLASS_IA),
                                    oss2.models.StorageTransition(days=2,
                                                                  storage_class=oss2.BUCKET_STORAGE_CLASS_ARCHIVE)]
        self.assertRaises(oss2.exceptions.InvalidArgument, self.bucket.put_bucket_lifecycle, BucketLifecycle([rule]))

        # 转储为IA(天数大于object过期时间)
        rule.storage_transitions = [oss2.models.StorageTransition(days=7,
                                                                  storage_class=oss2.BUCKET_STORAGE_CLASS_IA)]
        self.assertRaises(oss2.exceptions.InvalidArgument, self.bucket.put_bucket_lifecycle, BucketLifecycle([rule]))

        # 转储为STANDARD
        rule.storage_transitions = [oss2.models.StorageTransition(days=5,
                                                                  storage_class=oss2.BUCKET_STORAGE_CLASS_STANDARD)]
        self.assertRaises(oss2.exceptions.InvalidArgument, self.bucket.put_bucket_lifecycle, BucketLifecycle([rule]))

    def test_lifecycle_date(self):
        from oss2.models import LifecycleExpiration, LifecycleRule, BucketLifecycle

        rule = LifecycleRule(random_string(10), '中文前缀/',
                             status=LifecycleRule.ENABLED,
                             expiration=LifecycleExpiration(date=datetime.date(2016, 12, 25)))
        lifecycle = BucketLifecycle([rule])

        self.bucket.put_bucket_lifecycle(lifecycle)
        self.retry_assert(lambda: self.same_lifecycle(rule, self.bucket))

        self.bucket.delete_bucket_lifecycle()

    def test_lifecycle_created_before_date(self):
        from oss2.models import LifecycleExpiration, LifecycleRule, BucketLifecycle

        rule = LifecycleRule(random_string(10), '中文前缀/',
                             status=LifecycleRule.ENABLED,
                             expiration=LifecycleExpiration(created_before_date=datetime.date(2016, 12, 25)))
        lifecycle = BucketLifecycle([rule])

        self.bucket.put_bucket_lifecycle(lifecycle)
        self.retry_assert(lambda: self.same_lifecycle(rule, self.bucket))

        self.bucket.delete_bucket_lifecycle()

    def test_lifecycle_abort_multipart_upload_days(self):
        from oss2.models import LifecycleExpiration, LifecycleRule, BucketLifecycle, AbortMultipartUpload

        rule = LifecycleRule(random_string(10), '中文前缀/',
                             status=LifecycleRule.ENABLED,
                             expiration=LifecycleExpiration(created_before_date=datetime.date(2016, 12, 25)))

        rule.abort_multipart_upload = AbortMultipartUpload(days=356)

        lifecycle = BucketLifecycle([rule])

        self.bucket.put_bucket_lifecycle(lifecycle)
        self.retry_assert(lambda: self.same_lifecycle(rule, self.bucket))

        result = self.bucket.get_bucket_lifecycle()
        self.assertEqual(1, len(result.rules))
        self.assertEqual(356, result.rules[0].abort_multipart_upload.days)

        self.assertTrue(result.rules[0].tagging is None)

        self.bucket.delete_bucket_lifecycle()

    def test_lifecycle_abort_multipart_upload_date(self):
        from oss2.models import LifecycleExpiration, LifecycleRule, BucketLifecycle, AbortMultipartUpload

        rule = LifecycleRule(random_string(10), '中文前缀/',
                             status=LifecycleRule.ENABLED,
                             expiration=LifecycleExpiration(created_before_date=datetime.date(2016, 12, 25)))
        rule.abort_multipart_upload = AbortMultipartUpload(created_before_date=datetime.date(2016, 12, 20))

        lifecycle = BucketLifecycle([rule])

        self.bucket.put_bucket_lifecycle(lifecycle)

        wait_meta_sync()
        result = self.bucket.get_bucket_lifecycle()
        self.assertEqual(1, len(result.rules))
        self.assertEqual(datetime.date(2016, 12, 20), result.rules[0].abort_multipart_upload.created_before_date)
        
        self.assertTrue(result.rules[0].tagging is None)

        self.bucket.delete_bucket_lifecycle()

    def test_lifecycle_storage_transitions_mixed(self):
        from oss2.models import LifecycleExpiration, LifecycleRule, BucketLifecycle, StorageTransition

        rule = LifecycleRule(random_string(10), '中文前缀/',
                             status=LifecycleRule.ENABLED,
                             expiration=LifecycleExpiration(created_before_date=datetime.date(2016, 12, 25)))

        rule.storage_transitions = [StorageTransition(days=356, storage_class=oss2.BUCKET_STORAGE_CLASS_IA)]

        lifecycle = BucketLifecycle([rule])

        self.assertRaises(oss2.exceptions.InvalidRequest, self.bucket.put_bucket_lifecycle, lifecycle)

    def test_lifecycle_storage_transitions_days(self):
        from oss2.models import LifecycleExpiration, LifecycleRule, BucketLifecycle, StorageTransition

        rule = LifecycleRule(random_string(10), '中文前缀/',
                             status=LifecycleRule.ENABLED,
                             expiration=LifecycleExpiration(days=357))

        rule.storage_transitions = [StorageTransition(days=356, storage_class=oss2.BUCKET_STORAGE_CLASS_IA)]

        lifecycle = BucketLifecycle([rule])

        self.bucket.put_bucket_lifecycle(lifecycle)
        wait_meta_sync()
        result = self.bucket.get_bucket_lifecycle()
        self.assertEqual(1, len(result.rules))
        self.assertEqual(1, len(result.rules[0].storage_transitions))
        self.assertEqual(356, result.rules[0].storage_transitions[0].days)

        self.assertTrue(result.rules[0].tagging is None)

        self.bucket.delete_bucket_lifecycle()

    def test_lifecycle_storage_transitions_more_days(self):
        from oss2.models import LifecycleExpiration, LifecycleRule, BucketLifecycle, StorageTransition

        rule = LifecycleRule(random_string(10), '中文前缀/',
                             status=LifecycleRule.ENABLED,
                             expiration=LifecycleExpiration(days=357))

        rule.storage_transitions = [StorageTransition(days=355, storage_class=oss2.BUCKET_STORAGE_CLASS_IA),
                                    StorageTransition(days=356, storage_class=oss2.BUCKET_STORAGE_CLASS_ARCHIVE)]

        lifecycle = BucketLifecycle([rule])

        self.bucket.put_bucket_lifecycle(lifecycle)
        wait_meta_sync()
        result = self.bucket.get_bucket_lifecycle()
        self.assertEqual(1, len(result.rules))
        self.assertEqual(2, len(result.rules[0].storage_transitions))
        self.assertTrue(result.rules[0].tagging is None)
        if result.rules[0].storage_transitions[0].storage_class == oss2.BUCKET_STORAGE_CLASS_IA:
            self.assertEqual(355, result.rules[0].storage_transitions[0].days)
            self.assertEqual(356, result.rules[0].storage_transitions[1].days)
            self.assertEqual(oss2.BUCKET_STORAGE_CLASS_ARCHIVE, result.rules[0].storage_transitions[1].storage_class)
        else:
            self.assertEqual(356, result.rules[0].storage_transitions[0].days)
            self.assertEqual(356, result.rules[0].storage_transitions[1].days)
            self.assertEqual(oss2.BUCKET_STORAGE_CLASS_IA, result.rules[0].storage_transitions[1].storage_class)
        self.bucket.delete_bucket_lifecycle()

    def test_lifecycle_storage_transitions_date(self):
        from oss2.models import LifecycleExpiration, LifecycleRule, BucketLifecycle, StorageTransition

        rule = LifecycleRule(random_string(10), '中文前缀/',
                             status=LifecycleRule.ENABLED,
                             expiration=LifecycleExpiration(created_before_date=datetime.date(2016, 12, 25)))
        rule.storage_transitions = [StorageTransition(created_before_date=datetime.date(2016, 12, 20),
                                                      storage_class=oss2.BUCKET_STORAGE_CLASS_IA)]

        lifecycle = BucketLifecycle([rule])

        self.bucket.put_bucket_lifecycle(lifecycle)
        wait_meta_sync()
        result = self.bucket.get_bucket_lifecycle()
        self.assertEqual(1, len(result.rules))
        self.assertEqual(1, len(result.rules[0].storage_transitions))
        self.assertEqual(datetime.date(2016, 12, 20), result.rules[0].storage_transitions[0].created_before_date)

        self.assertTrue(result.rules[0].tagging is None)

        self.bucket.delete_bucket_lifecycle()

    def test_lifecycle_object_tagging(self):
        from oss2.models import LifecycleExpiration, LifecycleRule, BucketLifecycle, StorageTransition, Tagging, TaggingRule

        rule = LifecycleRule(random_string(10), 'aaaaaaaaaaa/',
                             status=LifecycleRule.ENABLED,
                             expiration=LifecycleExpiration(created_before_date=datetime.date(2016, 12, 25)))
        rule.storage_transitions = [StorageTransition(created_before_date=datetime.date(2016, 12, 20),
                                                      storage_class=oss2.BUCKET_STORAGE_CLASS_IA)]

        tagging_rule = TaggingRule()
        tagging_rule.add('test_key', 'test_value')
        tagging = Tagging(tagging_rule)

        rule.tagging = tagging

        lifecycle = BucketLifecycle([rule])

        self.bucket.put_bucket_lifecycle(lifecycle)
        wait_meta_sync()
        result = self.bucket.get_bucket_lifecycle()
        self.assertEqual(1, len(result.rules))
        self.assertEqual(1, len(result.rules[0].storage_transitions))
        self.assertEqual(datetime.date(2016, 12, 20), result.rules[0].storage_transitions[0].created_before_date)

        self.assertEqual(1, result.rules[0].tagging.tag_set.len())
        self.assertEqual('test_value', result.rules[0].tagging.tag_set.tagging_rule['test_key'])

        self.bucket.delete_bucket_lifecycle()


    def test_lifecycle_all_without_object_expiration(self):
        from oss2.models import LifecycleRule, BucketLifecycle, AbortMultipartUpload, StorageTransition

        rule = LifecycleRule(random_string(10), '中文前缀/',
                             status=LifecycleRule.ENABLED)

        rule.abort_multipart_upload = AbortMultipartUpload(days=356)
        rule.storage_transitions = [StorageTransition(days=356, storage_class=oss2.BUCKET_STORAGE_CLASS_IA)]

        lifecycle = BucketLifecycle([rule])

        self.bucket.put_bucket_lifecycle(lifecycle)
        wait_meta_sync()
        result = self.bucket.get_bucket_lifecycle()
        self.assertEqual(1, len(result.rules))
        self.assertEqual(356, result.rules[0].abort_multipart_upload.days)
        self.assertEqual(1, len(result.rules[0].storage_transitions))
        self.assertEqual(356, result.rules[0].storage_transitions[0].days)

        self.bucket.delete_bucket_lifecycle()

    def test_lifecycle_all(self):
        from oss2.models import LifecycleExpiration, LifecycleRule, BucketLifecycle, AbortMultipartUpload, StorageTransition

        rule = LifecycleRule(random_string(10), '中文前缀/',
                             status=LifecycleRule.ENABLED,
                             expiration=LifecycleExpiration(days=357))

        self.assertRaises(oss2.exceptions.ClientError,
                          LifecycleExpiration, days=356, created_before_date=datetime.date(2016, 12, 25))

        self.assertRaises(oss2.exceptions.ClientError,
                          AbortMultipartUpload, days=356, created_before_date=datetime.date(2016, 12, 25))

        self.assertRaises(oss2.exceptions.ClientError,
                          StorageTransition, days=356, created_before_date=datetime.date(2016, 12, 25))

        rule.abort_multipart_upload = AbortMultipartUpload(days=356)
        rule.storage_transitions = [StorageTransition(days=356, storage_class=oss2.BUCKET_STORAGE_CLASS_IA)]

        lifecycle = BucketLifecycle([rule])

        self.bucket.put_bucket_lifecycle(lifecycle)

        result = self.bucket.get_bucket_lifecycle()
        self.assertEqual(1, len(result.rules))
        self.assertEqual(356, result.rules[0].abort_multipart_upload.days)
        self.assertEqual(1, len(result.rules[0].storage_transitions))
        self.assertEqual(356, result.rules[0].storage_transitions[0].days)

        self.bucket.delete_bucket_lifecycle()

    def test_lifecycle_object_tagging_exceptions_wrong_key(self):

        from oss2.models import LifecycleExpiration, LifecycleRule, BucketLifecycle, StorageTransition, Tagging, TaggingRule

        rule = LifecycleRule(random_string(10), '中文前缀/',
                             status=LifecycleRule.ENABLED,
                             expiration=LifecycleExpiration(created_before_date=datetime.date(2016, 12, 25)))
        rule.storage_transitions = [StorageTransition(created_before_date=datetime.date(2016, 12, 20),
                                                      storage_class=oss2.BUCKET_STORAGE_CLASS_IA)]

        tagging = Tagging()
        
        tagging.tag_set.tagging_rule[129*'a'] = 'test'

        rule.tagging = tagging

        lifecycle = BucketLifecycle([rule])
        
        try:
            # do not return error,but the lifecycle rule doesn't take effect
            result = self.bucket.put_bucket_lifecycle(lifecycle)
        except oss2.exceptions.OssError:
            self.assertFalse(True, "put lifecycle with tagging should fail ,but success")
        
        del tagging.tag_set.tagging_rule[129*'a']

        tagging.tag_set.tagging_rule['%&'] = 'test'
        lifecycle.rules[0].tagging = tagging 
        try:
            # do not return error,but the lifecycle rule doesn't take effect
            result = self.bucket.put_bucket_lifecycle(lifecycle)
            self.assertFalse(True, "put lifecycle with tagging should fail ,but success")
        except oss2.exceptions.OssError:
            pass

    def test_lifecycle_object_tagging_exceptions_wrong_value(self):

        from oss2.models import LifecycleExpiration, LifecycleRule, BucketLifecycle, StorageTransition, Tagging, TaggingRule

        rule = LifecycleRule(random_string(10), '中文前缀/',
                             status=LifecycleRule.ENABLED,
                             expiration=LifecycleExpiration(created_before_date=datetime.date(2016, 12, 25)))
        rule.storage_transitions = [StorageTransition(created_before_date=datetime.date(2016, 12, 20),
                                                      storage_class=oss2.BUCKET_STORAGE_CLASS_IA)]

        tagging = Tagging()
        
        tagging.tag_set.tagging_rule['test'] = 257*'a'

        rule.tagging = tagging

        lifecycle = BucketLifecycle([rule])
        
        try:
            # do not return error,but the lifecycle rule doesn't take effect
            result = self.bucket.put_bucket_lifecycle(lifecycle)
        except oss2.exceptions.OssError:
            self.assertFalse(True, "put lifecycle with tagging should fail ,but success")

        tagging.tag_set.tagging_rule['test'] = ')%'
        rule.tagging = tagging
        lifecycle = BucketLifecycle([rule])
        try:
            # do not return error,but the lifecycle rule doesn't take effect
            result = self.bucket.put_bucket_lifecycle(lifecycle)
            self.assertFalse(True, "put lifecycle with tagging should fail ,but success")
        except oss2.exceptions.OssError:
            pass
    def test_lifecycle_object_tagging_exceptions_too_much_rules(self):

        from oss2.models import LifecycleExpiration, LifecycleRule, BucketLifecycle, StorageTransition, Tagging, TaggingRule

        rule = LifecycleRule(random_string(10), '中文前缀/',
                             status=LifecycleRule.ENABLED,
                             expiration=LifecycleExpiration(created_before_date=datetime.date(2016, 12, 25)))
        rule.storage_transitions = [StorageTransition(created_before_date=datetime.date(2016, 12, 20),
                                                      storage_class=oss2.BUCKET_STORAGE_CLASS_IA)]

        tagging = Tagging()
        for i in range(1, 20):
            key='test_key_'+str(i)
            value='test_value_'+str(i)
            tagging.tag_set.tagging_rule[key]=value

        
        rule.tagging = tagging

        lifecycle = BucketLifecycle([rule])
        
        try:
            # do not return error,but the lifecycle rule doesn't take effect
            result = self.bucket.put_bucket_lifecycle(lifecycle)
        except oss2.exceptions.OssError:
            self.assertFalse(True, "put lifecycle with tagging should fail ,but success")

    def test_cors(self):
        rule = oss2.models.CorsRule(allowed_origins=['*'],
                                    allowed_methods=['HEAD', 'GET'],
                                    allowed_headers=['*'],
                                    max_age_seconds=1000)
        cors = oss2.models.BucketCors([rule])

        self.bucket.put_bucket_cors(cors)
        wait_meta_sync()

        cors_got = self.bucket.get_bucket_cors()
        rule_got = cors_got.rules[0]

        self.assertEqual(rule.allowed_origins, rule_got.allowed_origins)
        self.assertEqual(rule.allowed_methods, rule_got.allowed_methods)
        self.assertEqual(rule.allowed_headers, rule_got.allowed_headers)
        self.assertEqual(rule.max_age_seconds, rule_got.max_age_seconds)

        self.bucket.delete_bucket_cors()
        self.bucket.delete_bucket_cors()
        wait_meta_sync()
        self.assertRaises(oss2.exceptions.NoSuchCors, self.bucket.get_bucket_cors)

    def test_bucket_stat(self):
        auth = oss2.Auth(OSS_ID, OSS_SECRET)
        bucket_name = OSS_BUCKET + "-test-stat"
        bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)

        bucket.create_bucket(oss2.BUCKET_ACL_PRIVATE)

        service = oss2.Service(auth, OSS_ENDPOINT)
        wait_meta_sync()
        self.retry_assert(lambda: bucket.bucket_name in (b.name for b in
                                                         service.list_buckets(prefix=bucket.bucket_name).buckets))

        key = 'a.txt'
        bucket.put_object(key, 'content')
        wait_meta_sync()

        result = bucket.get_bucket_stat()
        self.assertEqual(1, result.object_count)
        self.assertEqual(0, result.multi_part_upload_count)
        self.assertEqual(7, result.storage_size_in_bytes)

        bucket.delete_object(key)
        bucket.delete_bucket()

        wait_meta_sync()
        self.assertRaises(oss2.exceptions.NoSuchBucket, bucket.delete_bucket)

    def test_bucket_info(self):
        auth = oss2.Auth(OSS_ID, OSS_SECRET)
        bucket_name = OSS_BUCKET + "-test-info"
        bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)

        self.assertRaises(oss2.exceptions.NoSuchBucket, bucket.get_bucket_info)

        bucket.create_bucket(oss2.BUCKET_ACL_PRIVATE)

        service = oss2.Service(auth, OSS_ENDPOINT)
        wait_meta_sync()
        self.retry_assert(lambda: bucket.bucket_name in (b.name for b in
                                                         service.list_buckets(prefix=bucket.bucket_name).buckets))
        result = bucket.get_bucket_info()
        self.assertEqual(result.name, bucket.bucket_name)
        self.assertEqual(result.storage_class, oss2.BUCKET_STORAGE_CLASS_STANDARD)
        self.assertTrue(len(result.creation_date) > 0)
        self.assertTrue(len(result.intranet_endpoint) > 0)
        self.assertTrue(len(result.extranet_endpoint) > 0)
        self.assertTrue(len(result.owner.id) > 0)
        self.assertEqual(result.acl.grant, oss2.BUCKET_ACL_PRIVATE)
        self.assertIsNotNone(result.data_redundancy_type)
        self.assertIsNotNone(result.comment)
        self.assertEqual(result.bucket_encryption_rule.sse_algorithm, None)
        self.assertEqual(result.versioning_status, None)
        bucket.delete_bucket()

        wait_meta_sync()
        self.assertRaises(oss2.exceptions.NoSuchBucket, bucket.delete_bucket)

    def test_referer(self):
        referers = ['http://hello.com', 'mibrowser:home', '中文+referer', u'中文+referer']
        config = oss2.models.BucketReferer(True, referers)

        self.bucket.put_bucket_referer(config)
        wait_meta_sync()

        result = self.bucket.get_bucket_referer()

        self.assertTrue(result.allow_empty_referer)
        self.assertEqual(sorted(to_string(r) for r in referers), sorted(to_string(r) for r in result.referers))

    def test_location(self):
        result = self.bucket.get_bucket_location()
        self.assertTrue(result.location)

    def test_bucket_encryption_wrong(self):

        from oss2.models import ServerSideEncryptionRule

        self.assertRaises(oss2.exceptions.NoSuchServerSideEncryptionRule, self.bucket.get_bucket_encryption)

        rule = ServerSideEncryptionRule()
        rule.sse_algorithm = oss2.SERVER_SIDE_ENCRYPTION_AES256
        rule.kms_master_keyid = "test"

        self.assertRaises(oss2.exceptions.InvalidArgument,
                self.bucket.put_bucket_encryption, rule)

        rule.sse_algorithm = "random"
        rule.kms_master_keyid = ""
        self.assertRaises(oss2.exceptions.InvalidEncryptionAlgorithmError,
                self.bucket.put_bucket_encryption, rule)

        rule.sse_algorithm = oss2.SERVER_SIDE_ENCRYPTION_KMS 
        rule.kms_master_keyid = ""
        result = self.bucket.put_bucket_encryption(rule)
        self.assertEqual(int(result.status)/100, 2)

        rule.kms_master_keyid = None
        result = self.bucket.put_bucket_encryption(rule)
        self.assertEqual(int(result.status)/100, 2)

        result = self.bucket.get_bucket_encryption()
        self.assertEqual(result.sse_algorithm, oss2.SERVER_SIDE_ENCRYPTION_KMS)
        self.assertTrue(result.kms_master_keyid is None)

        result = self.bucket.delete_bucket_encryption()

        rule.sse_algorithm = oss2.SERVER_SIDE_ENCRYPTION_KMS
        rule.kms_master_keyid = "test_wrong"

        result = self.bucket.put_bucket_encryption(rule)
        self.assertEqual(int(result.status)/100, 2)

        result = self.bucket.get_bucket_encryption()
        self.assertEqual(result.sse_algorithm, oss2.SERVER_SIDE_ENCRYPTION_KMS)
        self.assertEqual(result.kms_master_keyid, "test_wrong")

        result = self.bucket.delete_bucket_encryption()

        self.assertEqual(int(result.status), 204)

    def test_bucket_encryption(self):

        from oss2.models import ServerSideEncryptionRule

        rule = ServerSideEncryptionRule()

        # AES256
        rule.sse_algorithm = oss2.SERVER_SIDE_ENCRYPTION_AES256
        rule.kms_master_keyid = ""

        result = self.bucket.put_bucket_encryption(rule)
        self.assertEqual(int(result.status)/100, 2)
    
        wait_meta_sync()

        result = self.bucket.get_bucket_info()
        self.assertEqual(result.bucket_encryption_rule.sse_algorithm, 'AES256')
        self.assertTrue(result.bucket_encryption_rule.kms_master_keyid is None)

        result = self.bucket.put_object("test", "test")
        self.assertEqual(int(result.status)/100, 2)
        
        result = self.bucket.get_object("test")
        self.assertEqual(int(result.status)/100, 2)

        self.assertEqual(b'test', result.read())

        result = self.bucket.delete_bucket_encryption()
        self.assertEqual(int(result.status), 204)

        # KMS
        rule.sse_algorithm = oss2.SERVER_SIDE_ENCRYPTION_KMS
        rule.kms_master_keyid = "123"

        result = self.bucket.put_bucket_encryption(rule)
        self.assertEqual(int(result.status)/100, 2)
    
        wait_meta_sync()

        result = self.bucket.get_bucket_info()
        self.assertEqual(result.bucket_encryption_rule.sse_algorithm, 'KMS')
        self.assertEqual(result.bucket_encryption_rule.kms_master_keyid, '123')

        result = self.bucket.delete_bucket_encryption()
        self.assertEqual(int(result.status), 204)

    def test_bucket_tagging(self):
        
        from oss2.models import Tagging, TaggingRule

        rule = TaggingRule()
        self.assertRaises(oss2.exceptions.ClientError, rule.add, 129*'a', 'test')
        self.assertRaises(oss2.exceptions.ClientError, rule.add, 'test', 257*'a')
        self.assertRaises(oss2.exceptions.ClientError, rule.add, None, 'test')
        self.assertRaises(oss2.exceptions.ClientError, rule.add, '', 'test')
        self.assertRaises(KeyError, rule.delete, 'not_exist')

        tagging = Tagging()
        tagging.tag_set.tagging_rule['%@abc'] = 'abc'
        tagging.tag_set.tagging_rule['123++'] = '++123%'

        try:
            result = self.bucket.put_bucket_tagging(tagging)
        except oss2.exceptions.OssError:
            self.assertFalse(True, 'should not get exception')
            pass

        result = self.bucket.get_bucket_tagging()
        tag_rule = result.tag_set.tagging_rule
        self.assertEqual(2, len(tag_rule))
        self.assertEqual('abc', tag_rule['%@abc'])
        self.assertEqual('++123%', tag_rule['123++'])

        result = self.bucket.delete_bucket_tagging()
        self.assertEqual(int(result.status), 204)

    def test_list_bucket_with_tagging(self):

        from oss2.models import Tagging, TaggingRule

        auth = oss2.Auth(OSS_ID, OSS_SECRET)
        service = oss2.Service(auth, OSS_ENDPOINT)

        bucket_name1 = OSS_BUCKET + "-test-with-tagging-1"
        bucket1 = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name1)

        bucket1.create_bucket(oss2.BUCKET_ACL_PRIVATE)

        wait_meta_sync()

        bucket_name2 = OSS_BUCKET + "-test-with-tagging-2"
        bucket2 = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name2)

        bucket2.create_bucket(oss2.BUCKET_ACL_PRIVATE)
        
        wait_meta_sync()

        rule = TaggingRule()
        rule.add('tagging_key_test_test1', 'value1')
        rule.add('tagging_key_test1', 'value1')

        tagging1 = Tagging(rule)
        try:
            result = bucket1.put_bucket_tagging(tagging1)
        except oss2.exceptions.OssError:
            self.assertFalse(True, 'should not get exception')
            pass

        rule = TaggingRule()
        rule.add('tagging_key2', 'value2')

        tagging2 = Tagging(rule)
        try:
            result = bucket2.put_bucket_tagging(tagging2)
        except oss2.exceptions.OssError:
            self.assertFalse(True, 'should not get exception')
            pass
        
        params = {}
        params['tag-key'] = 'tagging_key_test_test1'
        params['tag-value'] = 'value1'

        result = service.list_buckets(params=params)
        self.assertEqual(1, len(result.buckets))

        result = service.list_buckets()
        self.assertTrue(len(result.buckets) > 1)

        bucket1.delete_bucket()
        bucket2.delete_bucket()

    def test_bucket_policy(self):
        auth = oss2.Auth(OSS_ID, OSS_SECRET)
        bucket_name = OSS_BUCKET + "-test-policy"
        bucket = oss2.Bucket(auth, OSS_ENDPOINT, bucket_name)

        self.assertRaises(oss2.exceptions.NoSuchBucket, bucket.get_bucket_info)

        bucket.create_bucket(oss2.BUCKET_ACL_PRIVATE)

        service = oss2.Service(auth, OSS_ENDPOINT)
        wait_meta_sync()
        self.assertRaises(oss2.exceptions.NoSuchBucketPolicy, bucket.get_bucket_policy)

        policy=dict()
        policy["Version"] = "1"
        policy["Statement"] = []
        statement = dict()
        statement["Action"] = ["oss:PutObject"]
        statement["Effect"] = "Allow"
        statement["Resource"] = ["acs:oss:*:*:*/*"]
        policy["Statement"].append(statement)
        
        self.bucket.put_bucket_policy(json.dumps(policy))
        wait_meta_sync()

        result = self.bucket.get_bucket_policy()

        policy_json = json.loads(result.policy) 
        
        self.assertEqual(len(policy["Statement"]), len(policy_json["Statement"]))
        self.assertEqual(policy["Version"], policy_json["Version"])

        policy_resource = policy["Statement"][0]["Resource"][0]
        policy_json_resource = policy_json["Statement"][0]["Resource"][0]
        self.assertEqual(policy_resource, policy_json_resource)
        
        result = self.bucket.delete_bucket_policy()
        self.assertEqual(int(result.status)//100, 2)
        bucket.delete_bucket()

    def test_bucket_storage_capacity(self):
        auth = oss2.Auth(OSS_ID, OSS_SECRET)
        bucket = oss2.Bucket(auth, OSS_ENDPOINT, random_string(63).lower())
        bucket.create_bucket(oss2.BUCKET_ACL_PRIVATE)

        # test default storage capacity
        result = bucket.get_bucket_storage_capacity()
        self.assertEqual(result.storage_capacity, -1)

        #test set default capacity value -1
        user_qos = oss2.models.BucketUserQos(-1)
        bucket.set_bucket_storage_capacity(user_qos)
        result = bucket.get_bucket_storage_capacity()
        self.assertEqual(result.storage_capacity, -1)

        #set neagetive value other than -1
        user_qos = oss2.models.BucketUserQos(-2)
        self.assertRaises(oss2.exceptions.InvalidArgument, bucket.set_bucket_storage_capacity, user_qos) 

        #set positive value
        user_qos = oss2.models.BucketUserQos(100)
        resp = bucket.set_bucket_storage_capacity(user_qos)
        result = bucket.get_bucket_storage_capacity()
        self.assertEqual(result.storage_capacity, 100)

        bucket.delete_bucket()

    def test_malformed_xml(self):
        xml_input = '''<This is a bad xml></bad as I am>'''
        self.assertRaises(oss2.exceptions.MalformedXml, self.bucket.put_bucket_lifecycle, xml_input)

    def test_invalid_argument(self):
        rule = oss2.models.CorsRule(allowed_origins=['*'],
                                    allowed_methods=['HEAD', 'GET'],
                                    allowed_headers=['*'],
                                    max_age_seconds=-1)
        cors = oss2.models.BucketCors([rule])

        try:
            self.bucket.put_bucket_cors(cors)
        except oss2.exceptions.InvalidArgument as e:
            self.assertEqual(e.name, 'MaxAgeSeconds')
            self.assertEqual(e.value, '-1')

    def test_xml_input_output(self):
        xml_input1 = '''<?xml version="1.0" encoding="UTF-8"?>
                        <RefererConfiguration>
                          <AllowEmptyReferer>true</AllowEmptyReferer>
                          <RefererList>
                             <Referer>阿里云</Referer>
                          </RefererList>
                        </RefererConfiguration>'''
        xml_input2 = u'''<?xml version="1.0" encoding="UTF-8"?>
                         <RefererConfiguration>
                           <AllowEmptyReferer>true</AllowEmptyReferer>
                           <RefererList>
                             <Referer>阿里云</Referer>
                           </RefererList>
                         </RefererConfiguration>'''

        for input in [xml_input1, xml_input2]:
            self.bucket.put_bucket_referer(input)
            wait_meta_sync()

            resp = self.bucket._get_bucket_config(oss2.Bucket.REFERER)
            result = oss2.models.GetBucketRefererResult(resp)
            oss2.xml_utils.parse_get_bucket_referer(result, resp.read())

            self.assertEqual(result.allow_empty_referer, True)
            self.assertEqual(result.referers[0], to_string(u'阿里云'))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

import calendar
import base64
from random import choice

from oss2.exceptions import (ClientError, NotFound, NoSuchKey)
from oss2 import RsaProvider
from oss2 import http
from oss2 import headers

from .common import *
from .test_object import now
from oss2.compat import to_bytes


class TestCryptoObject(OssTestCase):
    # test cases for CryptoBucket
    def test_crypto_bucket_init_user_agent(self):
        crypto_bucket = self.rsa_crypto_bucket
        http_headers = None
        http_headers = http.CaseInsensitiveDict(http_headers)
        crypto_bucket._init_user_agent(http_headers)
        self.assertEqual(http_headers['User-Agent'], crypto_bucket.user_agent)

        user_agent = self.random_key()
        http_headers['User-Agent'] = user_agent
        crypto_bucket._init_user_agent(http_headers)
        self.assertEqual(http_headers['User-Agent'], user_agent + '/' + headers.OSS_ENCRYPTION_CLIENT)

    # 测试使用普通的Bucket读取加密的的对象数据
    def test_crypto_get_encrypted_object_with_bucket(self):
        crypto_bucket = choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])

        key = self.random_key('.js')
        content = random_bytes(1024)

        self.assertRaises(NotFound, self.bucket.head_object, key)

        lower_bound = now() - 60 * 16
        upper_bound = now() + 60 * 16

        def assert_result(result):
            self.assertEqual(result.content_length, len(content))
            self.assertEqual(result.content_type, 'application/javascript')
            self.assertEqual(result.object_type, 'Normal')

            self.assertTrue(result.last_modified > lower_bound)
            self.assertTrue(result.last_modified < upper_bound)

            self.assertTrue(result.etag)

        crypto_bucket.put_object(key, content)

        get_result = self.bucket.get_object(key)
        self.assertNotEqual(get_result.read(), content)
        assert_result(get_result)
        self.assertTrue(get_result.client_crc is not None)
        self.assertTrue(get_result.server_crc is not None)
        self.assertTrue(get_result.client_crc == get_result.server_crc)

        head_result = crypto_bucket.head_object(key)
        assert_result(head_result)

        self.assertEqual(get_result.last_modified, head_result.last_modified)
        self.assertEqual(get_result.etag, head_result.etag)

        crypto_bucket.delete_object(key)
        self.assertRaises(NoSuchKey, crypto_bucket.get_object, key)

    # 测试CryptoBucket普通put、get、delete、head等功能
    def test_crypto_object_basic(self):
        crypto_bucket = choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])

        key = self.random_key('.js')
        content = random_bytes(1024)

        self.assertRaises(NotFound, self.bucket.head_object, key)

        lower_bound = now() - 60 * 16
        upper_bound = now() + 60 * 16

        def assert_result(result):
            self.assertEqual(result.content_length, len(content))
            self.assertEqual(result.content_type, 'application/javascript')
            self.assertEqual(result.object_type, 'Normal')

            self.assertTrue(result.last_modified > lower_bound)
            self.assertTrue(result.last_modified < upper_bound)

            self.assertTrue(result.etag)

        crypto_bucket.put_object(key, content)

        get_result = crypto_bucket.get_object(key)
        self.assertEqual(get_result.read(), content)
        assert_result(get_result)
        self.assertTrue(get_result.client_crc is not None)
        self.assertTrue(get_result.server_crc is not None)
        self.assertTrue(get_result.client_crc == get_result.server_crc)

        head_result = crypto_bucket.head_object(key)
        assert_result(head_result)

        self.assertEqual(get_result.last_modified, head_result.last_modified)
        self.assertEqual(get_result.etag, head_result.etag)

        crypto_bucket.delete_object(key)
        self.assertRaises(NoSuchKey, crypto_bucket.get_object, key)

    def test_crypto_progress(self):
        crypto_bucket = choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        stats = {'previous': -1}

        def progress_callback(bytes_consumed, total_bytes):
            self.assertTrue(bytes_consumed <= total_bytes)
            self.assertTrue(bytes_consumed > stats['previous'])
            stats['previous'] = bytes_consumed

        key = self.random_key()
        content = random_bytes(2 * 1024 * 1024)

        # 上传内存中的内容
        stats = {'previous': -1}
        crypto_bucket.put_object(key, content, progress_callback=progress_callback)
        self.assertEqual(stats['previous'], len(content))

        # 下载到文件
        stats = {'previous': -1}
        filename = random_string(12) + '.txt'
        crypto_bucket.get_object_to_file(key, filename, progress_callback=progress_callback)
        self.assertEqual(stats['previous'], len(content))

        # 上传本地文件
        stats = {'previous': -1}
        crypto_bucket.put_object_from_file(key, filename, progress_callback=progress_callback)
        self.assertEqual(stats['previous'], len(content))

        # 下载到本地，采用iterator语法
        stats = {'previous': -1}
        result = crypto_bucket.get_object(key, progress_callback=progress_callback)
        content_got = b''
        for chunk in result:
            content_got += chunk
        self.assertEqual(stats['previous'], len(content))
        self.assertEqual(content, content_got)

        os.remove(filename)

    # 测试CryptoBucket range get功能
    def test_crypto_range_get(self):
        crypto_bucket = choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key()
        content = random_bytes(1024)

        crypto_bucket.put_object(key, content)

        get_result = crypto_bucket.get_object(key, byte_range=(None, None))
        self.assertEqual(get_result.read(), content)

        range_start = random.randint(0, 1024)
        get_result = crypto_bucket.get_object(key, byte_range=(range_start, None))
        self.assertEqual(get_result.read(), content[range_start:])

        # CryptoBucket由于需要range get的start值与block_size对齐，这种情况下不支持range_start为None的这种情况
        range_end = random.randint(0, 1024)
        self.assertRaises(ClientError, crypto_bucket.get_object, key, byte_range=(None, range_end))

        range_start = random.randint(0, 512)
        range_end = range_start + random.randint(0, 512)
        get_result = crypto_bucket.get_object(key, byte_range=(range_start, range_end))
        self.assertEqual(get_result.read(), content[range_start:range_end + 1])

    # 测试使用Bucket类的实例读取CryptoBucket类实例上传的对象
    def test_get_crypto_object_by_nomal_bucket(self):
        crypto_bucket = choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key('.js')
        content = random_bytes(1024)

        self.assertRaises(NotFound, crypto_bucket.head_object, key)

        result = crypto_bucket.put_object(key, content)
        self.assertTrue(result.status == 200)

        # self.assertRaises(ClientError, self.bucket.get_object, key)
        result = self.bucket.get_object(key)
        content_raw = result.read()
        self.assertEqual(len(content), len(content_raw))
        self.assertNotEqual(content, content_raw)

    # 测试使用CryptoBucket类读取Bucket类实例上传的对象
    def test_get_normal_object_by_crypto_bucket(self):
        crypto_bucket = choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key('.js')
        content = random_bytes(1024)

        self.assertRaises(NotFound, self.bucket.head_object, key)

        result = self.bucket.put_object(key, content)
        self.assertTrue(result.status == 200)

        get_result = crypto_bucket.get_object(key)
        self.assertEqual(get_result.read(), content)

    def test_crypto_get_object_with_url(self):
        crypto_bucket = choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key('.js')
        content = random_bytes(1024)

        result = crypto_bucket.put_object(key, content)
        self.assertTrue(result.status == 200)

        url = crypto_bucket.sign_url('GET', key, 3600)
        get_result = crypto_bucket.get_object_with_url(sign_url=url)
        self.assertEqual(get_result.read(), content)

    def test_crypto_put_object_with_url(self):
        crypto_bucket = choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key('.js')
        content = random_bytes(1024)

        url = crypto_bucket.sign_url('PUT', key, 3600)
        self.assertRaises(ClientError, crypto_bucket.put_object_with_url, url, content)

    def test_crypto_get_object_and_process(self):
        crypto_bucket = choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key('.jpg')
        result = crypto_bucket.put_object_from_file(key, "tests/example.jpg")
        self.assertTrue(result.status == 200)

        process = "image/resize,w_100"
        self.assertRaises(ClientError, crypto_bucket.get_object, key, process=process)

    def test_crypto_get_object_with_url_and_process(self):
        crypto_bucket = choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key('.jpg')
        result = crypto_bucket.put_object_from_file(key, "tests/example.jpg")
        self.assertTrue(result.status == 200)

        params = {oss2.Bucket.PROCESS: "image/resize,w_100"}
        url = crypto_bucket.sign_url('GET', key, 3600, params=params)
        self.assertRaises(ClientError, crypto_bucket.get_object_with_url, url)

    # 测试使用CryptoBucket类的append接口, 此时应该抛出异常
    def test_crypto_append_object(self):
        crypto_bucket = choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key('.log')

        self.assertRaises(NotFound, crypto_bucket.head_object, key)
        self.assertRaises(ClientError, crypto_bucket.append_object, key, 0, random_string(1024))

    def test_crypto_create_select_object_meta(self):
        crypto_bucket = choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key(".csv")
        result = crypto_bucket.put_object_from_file(key, 'tests/sample_data.csv')
        self.assertTrue(result.status == 200)

        self.assertRaises(ClientError, crypto_bucket.create_select_object_meta, key)

    def test_crypto_select_object(self):
        crypto_bucket = choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key(".csv")
        result = crypto_bucket.put_object_from_file(key, 'tests/sample_data.csv')
        self.assertTrue(result.status == 200)

        sql = "select Year, StateAbbr, CityName, PopulationCount from ossobject where CityName != ''"
        self.assertRaises(ClientError, crypto_bucket.select_object, key, sql)

    def test_crypto_process_object(self):
        crypto_bucket = choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key('.jpg')
        result = crypto_bucket.put_object_from_file(key, "tests/example.jpg")
        self.assertTrue(result.status == 200)

        dest_key = key[0:len(key) - 4] + '_dest.jpg'
        process = "image/resize,w_100|sys/saveas,o_{0},b_{1}".format(
            oss2.compat.to_string(base64.urlsafe_b64encode(oss2.compat.to_bytes(dest_key))),
            oss2.compat.to_string(base64.urlsafe_b64encode(oss2.compat.to_bytes(crypto_bucket.bucket_name))))
        self.assertRaises(ClientError, crypto_bucket.process_object, key, process)

    # 测试CryptoBucket类的Copy方法
    def test_copy_crypto_object(self):
        crypto_bucket = choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key('.js')
        content = random_bytes(1024)

        self.assertRaises(NotFound, crypto_bucket.head_object, key)

        headers = {'content-md5': oss2.utils.md5_string(content),
                   'content-length': str(len(content))}
        result = crypto_bucket.put_object(key, content, headers=headers)
        self.assertTrue(result.status == 200)

        target_key = key + "_target"
        result = crypto_bucket.copy_object(crypto_bucket.bucket_name, key, target_key)
        self.assertTrue(result.status == 200)

        def assert_result(result):
            self.assertEqual(result.content_length, len(content))
            self.assertEqual(result.content_type, 'application/javascript')
            self.assertEqual(result.object_type, 'Normal')
            self.assertTrue(result.etag)

        get_result = crypto_bucket.get_object(target_key)
        self.assertEqual(get_result.read(), content)
        assert_result(get_result)
        self.assertTrue(get_result.client_crc is not None)
        self.assertTrue(get_result.server_crc is not None)
        self.assertTrue(get_result.client_crc == get_result.server_crc)

    def test_crypto_object_incorrect_description(self):
        private_key_1 = RSA.generate(1024)
        public_key_1 = private_key_1.publickey()
        passphrase_1 = random_string(6)
        private_key_str_1 = RsaKey.exportKey(private_key_1, passphrase=passphrase_1)
        public_key_str_1 = RsaKey.exportKey(public_key_1, passphrase=passphrase_1)
        mat_desc_1 = {'key1': 'value1'}
        provider_1 = RsaProvider(key_pair={'private_key': private_key_str_1, 'public_key': public_key_str_1},
                                 passphrase=passphrase_1, mat_desc=mat_desc_1)
        crypto_bucket_1 = oss2.CryptoBucket(oss2.make_auth(OSS_ID, OSS_SECRET, OSS_AUTH_VERSION), OSS_ENDPOINT,
                                            OSS_BUCKET, crypto_provider=provider_1)
        private_key_2 = RSA.generate(2048)
        public_key_2 = private_key_2.publickey()
        passphrase_2 = random_string(6)
        private_key_str_2 = RsaKey.exportKey(private_key_2, passphrase=passphrase_2)
        public_key_str_2 = RsaKey.exportKey(public_key_2, passphrase=passphrase_2)
        mat_desc_2 = {'key2': 'value2'}
        provider_2 = RsaProvider(key_pair={'private_key': private_key_str_2, 'public_key': public_key_str_2},
                                 passphrase=passphrase_2, mat_desc=mat_desc_2)
        crypto_bucket_2 = oss2.CryptoBucket(oss2.make_auth(OSS_ID, OSS_SECRET, OSS_AUTH_VERSION), OSS_ENDPOINT,
                                            OSS_BUCKET, crypto_provider=provider_2)
        key = self.random_key('.js')
        content = random_bytes(1024)
        crypto_bucket_1.put_object(key, content)
        self.assertRaises(ClientError, crypto_bucket_2.get_object, key)

        # 口令未设置
        encryption_materials = oss2.EncryptionMaterials(mat_desc_1, key_pair={'private_key': private_key_str_1,
                                                                              'public_key': public_key_str_1})
        provider_2.add_encryption_materials(encryption_materials)
        self.assertRaises(ClientError, crypto_bucket_2.get_object, key)

        encryption_materials = oss2.EncryptionMaterials(mat_desc_1, key_pair={'private_key': private_key_str_1,
                                                                              'public_key': public_key_str_1},
                                                        passphrase=passphrase_1)
        provider_2.add_encryption_materials(encryption_materials)
        get_result = crypto_bucket_2.get_object(key)
        self.assertEqual(get_result.read(), content)

    def test_put_object_chunked(self):
        class FakeFileObj(object):
            def __init__(self, data, size):
                self.data = to_bytes(data)
                self.offset = 0
                self.size = size

            def read(self, amt=None):
                if self.offset >= self.size:
                    return to_bytes('')

                if amt is None or amt < 0:
                    bytes_to_read = self.size - self.offset
                else:
                    bytes_to_read = min(amt, self.size - self.offset)

                content = self.data[self.offset:self.offset + bytes_to_read]

                self.offset += bytes_to_read

                return content

        object_name = 'test-put-file-like-object-chunked'

        count = 1
        while count <= 100:
            count += 1
            cnt = random.randint(count, 1024)
            data = FakeFileObj(b'a' * cnt, count)
            self.rsa_crypto_bucket.put_object(object_name + str(count) + '.txt', data)
            get_result = self.rsa_crypto_bucket.get_object(object_name + str(count) + '.txt')
            self.assertEqual(get_result.read(), b'a' * count)


    '''
    # 测试CryptoBucket类的Copy方法, 并使用"REPLACE"模式修改meta
    def test_copy_crypto_object_with_replace_meta(self):
        crypto_bucket = choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key('.js')
        content = random_bytes(1024)

        self.assertRaises(NotFound, crypto_bucket.head_object, key)

        result = crypto_bucket.put_object(key, content)
        self.assertTrue(result.status == 200)

        meta_key = random_string(8)
        meta_value = random_string(16)

        target_key = key + "_target"
        headers = {'content-md5': oss2.utils.md5_string(content),
                   'content-length': str(len(content)),
                   'x-oss-metadata-directive': 'REPLACE',
                   'x-oss-meta-' + meta_key: meta_value}
        result = crypto_bucket.copy_object(crypto_bucket.bucket_name, key, target_key, headers=headers)
        self.assertTrue(result.status == 200)

        def assert_result(result):
            self.assertEqual(result.headers['x-oss-meta-' + meta_key], meta_value)
            self.assertEqual(result.content_length, len(content))
            self.assertEqual(result.content_type, 'application/javascript')
            self.assertEqual(result.object_type, 'Normal')
            self.assertTrue(result.etag)

        get_result = crypto_bucket.get_object(target_key)
        self.assertEqual(get_result.read(), content)
        assert_result(get_result)
        self.assertTrue(get_result.client_crc is not None)
        self.assertTrue(get_result.server_crc is not None)
        self.assertTrue(get_result.client_crc == get_result.server_crc)

    # 测试CryptoBucket类的Copy方法，修改加密元数据抛出异常
    def test_copy_crypto_object_with_replace_encryption_meta(self):
        crypto_bucket = choice([self.rsa_crypto_bucket, self.kms_crypto_bucket])
        key = self.random_key()
        dest_key = key + "-dest"

        content = random_bytes(1024)

        self.assertRaises(NotFound, crypto_bucket.head_object, key)

        result = crypto_bucket.put_object(key, content)
        self.assertTrue(result.status == 200)

        # replace mode, will raise excepiton InvalidEncryptionRequest
        headers = {'content-md5': oss2.utils.md5_string(content),
                   'content-length': str(len(content)),
                   'x-oss-metadata-directive': 'REPLACE',
                   'x-oss-meta-client-side-encryption-key': random_string(16)}
        self.assertRaises(oss2.exceptions.InvalidEncryptionRequest, crypto_bucket.copy_object,
                          self.bucket.bucket_name, key, dest_key, headers=headers)

        # copy mode, will ignore
        headers = {'content-md5': oss2.utils.md5_string(content),
                   'content-length': str(len(content)),
                   'x-oss-meta-client-side-encryption-key': random_string(16)}
        result = crypto_bucket.copy_object(crypto_bucket.bucket_name, key, dest_key, headers=headers)
        self.assertTrue(result.status == 200)

        result = crypto_bucket.get_object(key)
        self.assertEqual(result.read(), content)
    '''



# -*- coding: utf-8 -*-

import oss2
from .common import *
from oss2.headers import OSS_REQUEST_PAYER, OSS_OBJECT_TAGGING
from oss2.models import PAYER_REQUESTER, PartInfo, Tagging, TaggingRule
from oss2 import determine_part_size, SizedFileAdapter
import base64
import os


class TestObjectRequestPayment(OssTestCase):
    def setUp(self):
        OssTestCase.setUp(self)
        policy_text = ''
        policy_text += '{'
        policy_text += '"Version":"1",'
        policy_text += '"Statement":[{'
        policy_text += '"Action":["oss:*"],'
        policy_text += '"Effect":"Allow",'
        policy_text += '"Principal":["{0}"],'.format(OSS_PAYER_UID)
        policy_text += '"Resource": ["acs:oss:*:*:{0}","acs:oss:*:*:{0}/*"]'.format(OSS_BUCKET)
        policy_text += '}]}'
        self.bucket.put_bucket_policy(policy_text)

        # Enable bucket request payment
        result = self.bucket.put_bucket_request_payment(PAYER_REQUESTER)
        self.assertEqual(result.status, 200)

        self.payer_bucket = oss2.Bucket(oss2.Auth(OSS_PAYER_ID, OSS_PAYER_SECRET), OSS_ENDPOINT, OSS_BUCKET)

    def test_put_object(self):
        key = 'requestpayment-test-put-object'
        content = 'test-content'

        # Put object without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.put_object, key, content)

        # Put object with payer setting, should be successful.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        result = self.payer_bucket.put_object(key, content, headers=headers)
        self.assertEqual(result.status, 200)

        self.bucket.delete_object(key)

    def test_get_object(self):
        key = 'requestpayment-test-get-object'
        content = 'test-content'

        self.bucket.put_object(key, content)

        # Get object without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.get_object, key)

        # Put object with payer setting, should be successful.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        result = self.payer_bucket.get_object(key, headers=headers)
        self.assertEqual(result.status, 200)

        self.bucket.delete_object(key)

    def test_delete_object(self):
        key = 'requestpayment-test-delete-object'
        content = 'test-content'

        self.bucket.put_object(key, content)

        # Delete object without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.delete_object, key)

        # Delete object with payer setting, should be successful.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        result = self.payer_bucket.delete_object(key, headers=headers)
        self.assertEqual(result.status, 204)

    def test_batch_delete_object(self):
        key1 = 'requestpayment-test-batch-delete-object1'
        key2 = 'requestpayment-test-batch-delete-object2'
        content = 'test-content'

        objects = [key1, key2]

        self.bucket.put_object(key1, content)
        self.bucket.put_object(key2, content)

        # Delete objects without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.batch_delete_objects, objects)

        # Delete objects with payer setting, should be successful.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        result = self.payer_bucket.batch_delete_objects(objects, headers=headers)
        self.assertEqual(result.status, 200)

    def test_apend_object(self):
        key = 'requestpayment-test-apend-object'
        content1 = '123'
        content2 = '456'

        self.bucket.append_object(key, 0, content1)

        # Append object without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.append_object, key, 3, content2)

        # Append object with payer setting, should be successful.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        result = self.payer_bucket.append_object(key, 3,  content2, headers=headers)
        self.assertEqual(result.status, 200)

        self.bucket.delete_object(key)

    def test_put_object_acl(self):
        key = 'requestpayment-test-put-object-acl'
        content = '123'
        acl = oss2.OBJECT_ACL_PUBLIC_READ

        self.bucket.put_object(key, content)

        # Put object acl without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.put_object_acl, key, acl)

        # Put object acl with payer setting, should be successful.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        self.payer_bucket.put_object_acl(key, acl, headers=headers)

        # Check
        result = self.bucket.get_object_acl(key)
        self.assertEqual(result.acl, acl)

        self.bucket.delete_object(key)

    def test_get_object_acl(self):
        key = 'requestpayment-test-get-object-acl'
        content = '123'
        acl = oss2.OBJECT_ACL_PUBLIC_READ

        self.bucket.put_object(key, content)
        self.bucket.put_object_acl(key, acl)

        # Get object acl without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.get_object_acl, key)

        # Get object acl with payer setting, should be successful.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        result = self.payer_bucket.get_object_acl(key, headers=headers)
        self.assertEqual(result.acl, acl)

        self.bucket.delete_object(key)

    def test_put_symlink(self):
        target_object = 'requestpayment-test-put-symlink-object'
        symlink = 'requestpayment-test-put-symlink-symlink'
        content = '123'

        self.bucket.put_object(target_object, content)

        # Put symlink without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.put_symlink, target_object, symlink)

        # Put symlink with payer setting, should be successful.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        result = self.payer_bucket.put_symlink(target_object, symlink, headers=headers)
        self.assertEqual(result.status, 200)

        self.bucket.delete_object(target_object)
        self.bucket.delete_object(symlink)

    def test_get_symlink(self):
        target_object = 'requestpayment-test-get-symlink-object'
        symlink = 'requestpayment-test-get-symlink-symlink'
        content = '123'

        self.bucket.put_object(target_object, content)
        self.bucket.put_symlink(target_object, symlink)

        # Gett symlink without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.put_symlink, target_object, symlink)

        # Get symlink with payer setting, should be successful.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        result = self.payer_bucket.get_symlink(symlink, headers=headers)
        self.assertEqual(result.target_key, target_object)

        self.bucket.delete_object(target_object)
        self.bucket.delete_object(symlink)

    def test_copy_object(self):
        src_object_name = 'requestpayment-test-copy-object-src'
        dest_object_name = 'requestpayment-test-copy-object-dest'
        content = '123'
        bucket_name = self.bucket.bucket_name

        self.bucket.put_object(src_object_name, content)

        # Copy object without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.copy_object, bucket_name, 
                        src_object_name, dest_object_name)

        # Copy object with payer setting, should be successful.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        result = self.payer_bucket.copy_object(bucket_name, src_object_name, dest_object_name, headers=headers)
        self.assertEqual(result.status, 200)

        self.bucket.delete_object(src_object_name)
        self.bucket.delete_object(dest_object_name)

    def test_upload_part(self):
        key = 'requestpayment-test-upload-part-object'
        filename = key + '.txt'
        content = b'1' * 1024 * 1024

        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"

        content = random_bytes(1024 * 1024)

        with open(filename, 'wb') as f:
            f.write(content)

        total_size = os.path.getsize(filename)
        # Set part size
        part_size = determine_part_size(total_size, preferred_size=(100*1024))

        # Init multipart without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.init_multipart_upload, key)

        # Init multipart with payer setting, should be successful.
        upload_id = self.payer_bucket.init_multipart_upload(key, headers=headers).upload_id
        parts = []

        # Upload part without payer setting, should be failed.
        with open(filename, 'rb') as fileobj:
            part_number = 1
            num_to_upload = part_size
            self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.upload_part, key, upload_id, part_number,
                                            SizedFileAdapter(fileobj, num_to_upload))

        # Upload part with payer setting, should be successful.
        with open(filename, 'rb') as fileobj:
            part_number = 1
            offset = 0
            while offset < total_size:
                num_to_upload = min(part_size, total_size - offset)
                result = self.payer_bucket.upload_part(key, upload_id, part_number,
                                            SizedFileAdapter(fileobj, num_to_upload), headers=headers)
                parts.append(PartInfo(part_number, result.etag))

                offset += num_to_upload
                part_number += 1

        # Complete multipart upload without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.complete_multipart_upload, key, upload_id, parts)

        # Complete multipart upload with payer setting, should be successful.
        result = self.payer_bucket.complete_multipart_upload(key, upload_id, parts, headers=headers)
        self.assertEqual(result.status, 200)

        self.bucket.delete_object(key)
        os.remove(filename)

    def test_upload_part_copy(self):
        src_object_name = 'requestpayment-test-upload-part-copy-src'
        dest_object_name = 'requestpayment-test-upload-part-copy-dest'
        content = b'a' * 1024 * 1024

        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"

        self.bucket.put_object(src_object_name, content)        

        # Get src object size
        head_info = self.bucket.head_object(src_object_name)
        total_size = head_info.content_length
        self.assertEqual(total_size, 1024 * 1024)

        # Set part size
        part_size = determine_part_size(total_size, preferred_size=(100*1024))

        upload_id = self.payer_bucket.init_multipart_upload(dest_object_name, headers=headers).upload_id
        parts = []

        # Upload part copy without payer setting, should be failed.
        part_number = 1
        offset = 0
        num_to_upload = min(part_size, total_size - offset)
        end = offset + num_to_upload - 1;
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.upload_part_copy, self.payer_bucket.bucket_name, 
                            src_object_name, (offset, end), dest_object_name, upload_id, part_number)

        # Upload part copy with payer setting, should be successful.
        part_number = 1
        offset = 0
        while offset < total_size:
            num_to_upload = min(part_size, total_size - offset)
            end = offset + num_to_upload - 1;
            result = self.payer_bucket.upload_part_copy(self.payer_bucket.bucket_name, src_object_name, (offset, end), 
                                dest_object_name, upload_id, part_number, headers=headers)

            parts.append(PartInfo(part_number, result.etag))

            offset += num_to_upload
            part_number += 1

        # Complete multipart upload with payer setting, should be successful.
        result = self.payer_bucket.complete_multipart_upload(dest_object_name, upload_id, parts, headers=headers)

        self.bucket.delete_object(src_object_name)
        self.bucket.delete_object(dest_object_name)

    def test_resumable_upload(self):
        small_object = 'requestpayment-test-resumable-upload-small-object'
        big_object = 'requestpayment-test-resumable-upload-big-object'

        # Create tmp file smaller than multipart_threshold
        file_name = self._prepare_temp_file_with_size(150 * 1024)

        # Resumale upload small object without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, oss2.resumable_upload, self.payer_bucket, small_object, file_name, 
                        multipart_threshold=(200*1024), num_threads=2, part_size=(100*1024))

        # Resumale upload small object with payer setting, should be successful.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        result = oss2.resumable_upload(self.payer_bucket, small_object, file_name, 
                        multipart_threshold=(200*1024), num_threads=2, part_size=(100*1024), headers=headers)
        self.assertEqual(result.status, 200)
        self.bucket.delete_object(small_object)

        # Start big file test
        # Create big file bigger than multipart_threshold
        file_name = self._prepare_temp_file_with_size(11 *1024 * 1024)

        # Resumale upload big object without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, oss2.resumable_upload, self.payer_bucket, big_object, file_name, 
                        multipart_threshold=(200*1024), num_threads=2, part_size=(100*1024))

        # Resumale upload big object with payer setting and tagging setting, should be successful.
        key1 = 'key1'
        value1 = 'value2'

        key2 = 'key2'
        value2 = 'value2'

        tag_str = key1 + '=' + value1
        tag_str += '&' + key2 + '=' + value2

        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        headers[OSS_OBJECT_TAGGING] = tag_str
        result = oss2.resumable_upload(self.payer_bucket, big_object, file_name, 
                    multipart_threshold=(200*1024), num_threads=2, part_size=(100*1024), headers=headers)
        self.assertEqual(result.status, 200)

        # Check object size
        head_info = self.bucket.head_object(big_object)
        total_size = head_info.content_length
        self.assertEqual(total_size, (11 * 1024 * 1024))

        # Check tagging
        result = self.bucket.get_object_tagging(big_object)
        self.assertEqual(2, result.tag_set.len())
        tagging_rule = result.tag_set.tagging_rule
        self.assertEqual(value1, tagging_rule[key1])
        self.assertEqual(value2, tagging_rule[key2])

        self.bucket.delete_object(big_object)

    def test_resumable_down(self):
        small_object = 'requestpayment-test-resumable-down-small-object'
        content1 = b'a' * (150 * 1024)
        big_object = 'requestpayment-test-resumable-down-big-object'
        content2 = b'a' * (500 * 1024)
        file_name = small_object + '.txt'

        self.bucket.put_object(small_object, content1)
        self.bucket.put_object(big_object, content2)

        # Resumale down small object without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.ServerError, oss2.resumable_download, self.payer_bucket, small_object, file_name, 
                    multiget_threshold=(200*1024), num_threads=2, part_size=(1024*1024))

        # Resumale down small object with payer setting, should be successful.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        oss2.resumable_download(self.payer_bucket, small_object, file_name, 
                    multiget_threshold=(200*1024), num_threads=2, part_size=(1024*1024), headers=headers)

        # Check file size
        file_size = os.stat(file_name).st_size
        self.assertEqual(file_size, (150*1024))

        os.remove(file_name)
        self.bucket.delete_object(small_object)

        file_name = big_object + '.txt'

        # Resumale down big object without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.ServerError, oss2.resumable_download, self.payer_bucket, big_object, file_name, 
                    multiget_threshold=(200*1024), num_threads=2, part_size=(1024*1024))

        # Resumale down big object with payer setting, should be successful.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        oss2.resumable_download(self.payer_bucket, big_object, file_name, 
                    multiget_threshold=(200*1024), num_threads=2, part_size=(1024*1024), headers=headers)

        # Check file size
        file_size = os.stat(file_name).st_size
        self.assertEqual(file_size, (500*1024))

        os.remove(file_name)
        self.bucket.delete_object(big_object)

    def test_resumable_down1(self):
        key = 'requestpayment-test-resumable-down-object'
        content = b'a' * (2 * 1024 * 1024)
        file_name = key + '.txt'

        self.bucket.put_object(key, content)

        # Resumale down object smaller than multiget_threshold without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.ServerError, oss2.resumable_download, self.payer_bucket, key, file_name, 
                    multiget_threshold=(3*1024*1024), num_threads=2, part_size=(100*1024))

        # Resumale down object smaller than multiget_threshold with payer setting, should be successful.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        oss2.resumable_download(self.payer_bucket, key, file_name, 
                    multiget_threshold=(3*1024*1024), num_threads=2, part_size=(100*1024), headers=headers)

        # Check file size
        file_size = os.stat(file_name).st_size
        self.assertEqual(file_size, (2*1024*1024))

        os.remove(file_name)

        # Resumale down object bigger than multiget_threshold without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.ServerError, oss2.resumable_download, self.payer_bucket, key, file_name, 
                    multiget_threshold=(500*1024), num_threads=2, part_size=(100*1024))

        # Resumale down object bigger than multiget_threshold with payer setting, should be successful.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        oss2.resumable_download(self.payer_bucket, key, file_name, 
                    multiget_threshold=(500*1024), num_threads=2, part_size=(100*1024), headers=headers)

        # Check file size
        file_size = os.stat(file_name).st_size
        self.assertEqual(file_size, (2*1024*1024))

        os.remove(file_name)
        self.bucket.delete_object(key)

    def test_head_object(self):
        key = 'requestpayment-test-head-object'
        content = '123'

        self.bucket.put_object(key, content)

        # Head object without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.ServerError, self.payer_bucket.head_object, key)

        # Head object with payer setting, should be successful.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        result = self.payer_bucket.head_object(key, headers=headers)
        self.assertEqual(result.status, 200)

        self.bucket.delete_object(key)

    def test_get_object_meta(self):
        key = 'requestpayment-test-get-object-meta'
        content = '123'

        self.bucket.put_object(key, content)

        # Get object meta without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.get_object_meta, key)

        # Get object meta with payer setting, should be successful.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        result = self.payer_bucket.get_object_meta(key, headers=headers)
        self.assertEqual(result.status, 200)

        self.bucket.delete_object(key)

    def test_dose_exists_object(self):
        key = 'requestpayment-test-dose-exist-object'
        content = 'test-content'

        self.bucket.put_object(key, content)

        # Test dose exist object without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.object_exists, key)

        # Test dose exist object with payer setting, should be successful.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        is_exist = self.payer_bucket.object_exists(key, headers=headers)
        self.assertEqual(is_exist, True)

        self.bucket.delete_object(key)

    def test_update_object_meta(self):
        key = 'requestpayment-test-update-object-meta'
        content = 'test-content'

        self.bucket.put_object(key, content)

        headers = dict()
        headers['Content-Type'] = 'whatever'
        # Test dose exist object without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.update_object_meta, key, headers=headers)

        # Test dose exist object with payer setting, should be successful.
        headers[OSS_REQUEST_PAYER] = 'requester'
        self.payer_bucket.update_object_meta(key, headers=headers)

        self.bucket.delete_object(key)

    def test_restore_object(self):
        key = 'requestpayment-test-restore-object'
        content = 'test-content'
        headers = dict()
        headers['x-oss-storage-class'] = oss2.BUCKET_STORAGE_CLASS_ARCHIVE

        # Check object's storage class
        self.bucket.put_object(key, content, headers)
        meta = self.bucket.head_object(key)
        self.assertEqual(meta.resp.headers['x-oss-storage-class'], oss2.BUCKET_STORAGE_CLASS_ARCHIVE)

        # Restore object without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.restore_object, key)

        # Check object's storage class
        self.bucket.put_object(key, content, headers)
        meta = self.bucket.head_object(key)
        self.assertEqual(meta.resp.headers['x-oss-storage-class'], oss2.BUCKET_STORAGE_CLASS_ARCHIVE)

        # Restore object with payer setting, should be successful.
        headers=dict()
        headers[OSS_REQUEST_PAYER] = 'requester'
        self.payer_bucket.restore_object(key, headers=headers)

        self.bucket.delete_object(key)

    def test_abort_multipart_upload(self):
        key = 'requestpayment-abort-multipart-upload'
        
        upload_id = self.bucket.init_multipart_upload(key).upload_id

        # Abort multipart without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.abort_multipart_upload, key, upload_id)

        # Abort multipartwith payer setting, should be successful.
        headers=dict()
        headers[OSS_REQUEST_PAYER] = 'requester'
        result = self.payer_bucket.abort_multipart_upload(key, upload_id, headers=headers)
        self.assertEqual(result.status, 204)

    def test_process_object(self):
        key = "test-process-object.jpg"
        result = self.bucket.put_object_from_file(key, "tests/example.jpg")
        self.assertEqual(result.status, 200)
        dest_key = "test-process-object-dest.jpg"
        
        process = "image/resize,w_100|sys/saveas,o_{0},b_{1}".format(
            oss2.compat.to_string(base64.urlsafe_b64encode(oss2.compat.to_bytes(dest_key))),
            oss2.compat.to_string(base64.urlsafe_b64encode(oss2.compat.to_bytes(self.bucket.bucket_name))))

        # Process object without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.process_object, key, process)

        # Process object with payer setting, should be successful.
        headers=dict()
        headers[OSS_REQUEST_PAYER] = 'requester'
        result = self.payer_bucket.process_object(key, process, headers=headers)
        self.assertEqual(result.status, 200)

        result = self.bucket.object_exists(dest_key)
        self.assertEqual(result, True)

        self.bucket.delete_object(key)
        self.bucket.delete_object(dest_key)

    def test_object_tagging(self):
        key = 'requestpayment-test-put-get-delete-object-tagging'
        content = 'test-content'

        self.bucket.put_object(key, content)

        rule = TaggingRule()
        rule.add('key1', 'value1')
        rule.add('key2', 'value2')
        tagging = Tagging(rule)

        # Put object tagging without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.put_object_tagging, key, tagging)

        # Put object tagging with payer setting, should be successful.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        result = self.payer_bucket.put_object_tagging(key, tagging, headers=headers)
        self.assertEqual(result.status, 200)

        # Get object tagging without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.get_object_tagging, key)

        # Get object tagging with payer setting, should be successful.

        result = self.payer_bucket.get_object_tagging(key, headers=headers)
        self.assertEqual(len(result.tag_set.tagging_rule), 2)

        # Delete object tagging without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.delete_object_tagging, key)

        # Delete object tagging with payer setting, should be successful.
        result = self.payer_bucket.delete_object_tagging(key, headers=headers)
        self.assertEqual(result.status, 204)

        self.bucket.delete_object(key)

    def test_list_objects(self):
        # List object without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.list_objects)

        # List objects with payer setting, should be successful.
        headers=dict()
        headers[OSS_REQUEST_PAYER] = 'requester'
        result = self.payer_bucket.list_objects(headers=headers)
        self.assertEqual(result.status, 200)

    def test_list_parts(self):
        key = 'requestpayment-list-parts'

        upload_id = self.bucket.init_multipart_upload(key).upload_id

        # Abort multipart without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.list_parts, key, upload_id)

        # Abort multipartwith payer setting, should be successful.
        headers=dict()
        headers[OSS_REQUEST_PAYER] = 'requester'
        result = self.payer_bucket.list_parts(key, upload_id, headers=headers)
        self.assertEqual(result.status, 200)

    def test_list_mulitpart_uploads(self):
        # List multipart uploads without payer setting, should be failed.
        self.assertRaises(oss2.exceptions.AccessDenied, self.payer_bucket.list_multipart_uploads)

        # List multipart uploads with payer setting, should be successful.
        headers=dict()
        headers[OSS_REQUEST_PAYER] = 'requester'
        result = self.payer_bucket.list_multipart_uploads(headers=headers)
        self.assertEqual(result.status, 200)

    def test_object_iterator(self):
        # ObjectIterator without payer setting, should be failed.
        access_err_flag = False
        try:
            obj_iter = oss2.ObjectIterator(self.payer_bucket)
            for obj in obj_iter:
                pass
        except oss2.exceptions.AccessDenied:
            access_err_flag = True

        self.assertEqual(access_err_flag, True)

        # ObjectIterator with payer setting, should be failed.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        obj_iter = oss2.ObjectIterator(self.payer_bucket, headers=headers)
        for obj in obj_iter:
            pass

    def test_multipart_iterator(self):
        # MultipartUploadIterator without payer setting, should be failed.
        access_err_flag = False
        try:
            up_iter = oss2.MultipartUploadIterator(self.payer_bucket)
            for up in up_iter:
                pass
        except oss2.exceptions.AccessDenied:
            access_err_flag = True

        self.assertEqual(access_err_flag, True)

        # MultipartUploadIterator with payer setting, should be successful.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        up_iter = oss2.MultipartUploadIterator(self.payer_bucket, headers=headers)
        for up in up_iter:
            pass

    def test_object_upload_iterator(self):
        key = 'requestpayment-test-object-upload-iterator'
        content = 'test-content'
        self.bucket.put_object(key, content)

        # ObjectUploadIterator without payer setting, should be failed.
        access_err_flag = False
        try:
            up_iter = oss2.ObjectUploadIterator(self.payer_bucket, key)
            for up in up_iter:
                pass
        except oss2.exceptions.AccessDenied:
            access_err_flag = True

        self.assertEqual(access_err_flag, True)

        # ObjectUploadIterator with payer setting, should be successful.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        up_iter = oss2.ObjectUploadIterator(self.payer_bucket, key, headers=headers)
        for up in up_iter:
            pass

        self.bucket.delete_object(key)

    def test_part_iterator(self):
        key = 'requestpayment-test-object-upload-iterator'
        upload_id = self.bucket.init_multipart_upload(key).upload_id

        # PartIterator without payer setting, should be failed.
        access_err_flag = False
        try:
            up_iter = oss2.PartIterator(self.payer_bucket, key, upload_id)
            for up in up_iter:
                pass
        except oss2.exceptions.AccessDenied:
            access_err_flag = True

        self.assertEqual(access_err_flag, True)

        # PartIterator with payer setting, should be successful.
        headers = dict()
        headers[OSS_REQUEST_PAYER] = "requester"
        up_iter = oss2.PartIterator(self.payer_bucket, key, upload_id, headers=headers)
        for up in up_iter:
            pass

    def test_put_object_with_signed_url(self):
        key = 'request-payment-test-put-object-signed-url'
        file_name = self._prepare_temp_file_with_size(1024)

        params = dict()
        params[OSS_REQUEST_PAYER] = "requester" 
        url = self.payer_bucket.sign_url('PUT', key, 60, params=params)
        self.payer_bucket.put_object_with_url_from_file(url, file_name)

    def test_get_object_with_signed_url(self):
        key = 'request-payment-test-get-object-signed-url'
        content = b'a' * 1024
        file_name = key + '.txt'

        result = self.bucket.put_object(key, content);
        self.assertEqual(result.status, 200)

        params = dict()
        params[OSS_REQUEST_PAYER] = "requester"
        url = self.payer_bucket.sign_url('GET', key, 60, params=params)
        result = self.payer_bucket.get_object_with_url_to_file(url, file_name)

        os.remove(file_name)
        self.bucket.delete_object(key)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

import oss2

from functools import partial
from mock import patch

from unittests.common import *


class TestMultipart(OssTestCase):
    @patch('oss2.Session.do_request')
    def test_init(self, do_request):
        request_text = '''POST /uosvelpvgjwtxaciqtxoplnx?uploads= HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
Content-Length: 0
date: Sat, 12 Dec 2015 00:35:55 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
Accept: */*
authorization: OSS ZCDmm7TPZKHtx77j:2OVoUTO7rFeeGlpXH9M6ZMuh7d8=
'''
        response_text = '''HTTP/1.1 200 OK
Server: AliyunOSS
Date: Sat, 12 Dec 2015 00:35:55 GMT
Content-Type: application/xml
Content-Length: 232
Connection: keep-alive
x-oss-request-id: 566B6BEB1BA604C27DD43805

<?xml version="1.0" encoding="UTF-8"?>
<InitiateMultipartUploadResult>
  <Bucket>ming-oss-share</Bucket>
  <Key>uosvelpvgjwtxaciqtxoplnx</Key>
  <UploadId>97BD544A65DB46F9A8735C93917A960F</UploadId>
</InitiateMultipartUploadResult>
'''

        req_info = mock_response(do_request, response_text)
        result = bucket().init_multipart_upload('uosvelpvgjwtxaciqtxoplnx')

        self.assertRequest(req_info, request_text)
        self.assertEqual(result.upload_id, '97BD544A65DB46F9A8735C93917A960F')

    @patch('oss2.Session.do_request')
    def test_upload_part(self, do_request):
        content = random_bytes(1024)

        request_text = '''PUT /tmmzgvvmsgesihfo?partNumber=3&uploadId=41337E94168A4E6F918C3D6CAAFADCCD HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
Content-Length: 1024
date: Sat, 12 Dec 2015 00:35:59 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
Accept: */*
authorization: OSS ZCDmm7TPZKHtx77j:3+h0rLBaA3gPrM4iZoFSyQZn2ts=

{0}'''.format(oss2.to_string(content))

        response_text = '''HTTP/1.1 200 OK
Server: AliyunOSS
Date: Sat, 12 Dec 2015 00:35:59 GMT
Content-Length: 0
Connection: keep-alive
x-oss-request-id: 566B6BEF6078C0E44874A4AD
x-oss-hash-crc64ecma: {0}
ETag: "DF1F9DE8F39BDE03716AC8D425589A5A"'''.format(calc_crc(content))

        req_info = mock_response(do_request, response_text)
        result = bucket().upload_part('tmmzgvvmsgesihfo', '41337E94168A4E6F918C3D6CAAFADCCD', 3, content)

        self.assertRequest(req_info, request_text)
        self.assertEqual(req_info.data, content)

        self.assertEqual(result.etag, 'DF1F9DE8F39BDE03716AC8D425589A5A')

    @patch('oss2.Session.do_request')
    def test_upload_part_copy(self, do_request):
        request_text = '''PUT /pasncdoyuvuvuiyewfsobdwn?partNumber=1&uploadId=65484B78EF3846298B8E2DC1643F8F37 HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
x-oss-copy-source: /ming-oss-share/pasncdoyuvuvmgqtkchhuosw
Content-Length: 0
x-oss-copy-source-range: bytes=0-102399
date: Sat, 12 Dec 2015 00:36:25 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
Accept: */*
authorization: OSS ZCDmm7TPZKHtx77j:wbO3Klw0f6pMPy2lBDZqNtgZ9EY='''

        response_text = '''HTTP/1.1 200 OK
Server: AliyunOSS
Date: Sat, 12 Dec 2015 00:36:26 GMT
Content-Type: application/xml
Content-Length: 180
Connection: keep-alive
x-oss-request-id: 566B6C09D5A340D61A73D677
Content-Range: bytes 0-102399/204800
ETag: "4DE8075FB607DF4D13FBC480EA488EFA"

<?xml version="1.0" encoding="UTF-8"?>
<CopyPartResult>
  <LastModified>2015-12-12T00:36:26.000Z</LastModified>
  <ETag>"4DE8075FB607DF4D13FBC480EA488EFA"</ETag>
</CopyPartResult>'''

        req_info = mock_response(do_request, response_text)

        result = bucket().upload_part_copy('ming-oss-share', 'pasncdoyuvuvmgqtkchhuosw', (0, 102399),
                                           'pasncdoyuvuvuiyewfsobdwn', '65484B78EF3846298B8E2DC1643F8F37', 1)

        self.assertRequest(req_info, request_text)

        self.assertEqual(result.etag, '4DE8075FB607DF4D13FBC480EA488EFA')
        self.assertEqual(result.request_id, '566B6C09D5A340D61A73D677')

    @patch('oss2.Session.do_request')
    def test_complete(self, do_request):
        from oss2.models import PartInfo

        parts = list()
        parts.append(PartInfo(1, '4DE8075FB607DF4D13FBC480EA488EFA'))
        parts.append(PartInfo(2, 'AF947EC157726CEA88ED83B3C989063B'))

        request_text = '''POST /pasncdoyuvuvuiyewfsobdwn?uploadId=65484B78EF3846298B8E2DC1643F8F37 HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
Content-Length: 223
date: Sat, 12 Dec 2015 00:36:26 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
Accept: */*
authorization: OSS ZCDmm7TPZKHtx77j:TgjWAumJAl8dDr0yqWHOyqqwrd0=

<CompleteMultipartUpload><Part><PartNumber>1</PartNumber><ETag>"4DE8075FB607DF4D13FBC480EA488EFA"</ETag></Part><Part><PartNumber>2</PartNumber><ETag>"AF947EC157726CEA88ED83B3C989063B"</ETag></Part></CompleteMultipartUpload>'''

        response_text = '''HTTP/1.1 200 OK
Server: AliyunOSS
Date: Sat, 12 Dec 2015 00:36:26 GMT
Content-Type: application/xml
Content-Length: 327
Connection: keep-alive
x-oss-request-id: 566B6C0A05200A20B174994F
ETag: "1C787C506EABFB9B45EAAA8DB039F4B2-2"

<?xml version="1.0" encoding="UTF-8"?>
<CompleteMultipartUploadResult>
  <Location>http://ming-oss-share.oss-cn-hangzhou.aliyuncs.com/pasncdoyuvuvuiyewfsobdwn</Location>
  <Bucket>ming-oss-share</Bucket>
  <Key>pasncdoyuvuvuiyewfsobdwn</Key>
  <ETag>"1C787C506EABFB9B45EAAA8DB039F4B2-2"</ETag>
</CompleteMultipartUploadResult>'''

        req_info = mock_response(do_request, response_text)

        result = bucket().complete_multipart_upload('pasncdoyuvuvuiyewfsobdwn', '65484B78EF3846298B8E2DC1643F8F37', parts)

        self.assertRequest(req_info, request_text)
        self.assertEqual(result.etag, '1C787C506EABFB9B45EAAA8DB039F4B2-2')

    @patch('oss2.Session.do_request')
    def test_abort(self, do_request):
        request_text = '''DELETE /uosvelpvgjwtxaciqtxoplnx?uploadId=97BD544A65DB46F9A8735C93917A960F HTTP/1.1
Host: ming-oss-share.oss-cn-hangzhou.aliyuncs.com
Accept-Encoding: identity
Connection: keep-alive
Content-Length: 0
date: Sat, 12 Dec 2015 00:35:56 GMT
User-Agent: aliyun-sdk-python/2.0.2(Windows/7/;3.3.3)
Accept: */*
authorization: OSS ZCDmm7TPZKHtx77j:anUcRNyx/g9BxU8xplJHn2BcRvQ='''

        response_text = '''HTTP/1.1 204 No Content
Server: AliyunOSS
Date: Sat, 12 Dec 2015 00:35:56 GMT
Content-Length: 0
Connection: keep-alive
x-oss-request-id: 566B6BEC1BA604C27DD438F8'''

        req_info = mock_response(do_request, response_text)
        bucket().abort_multipart_upload('uosvelpvgjwtxaciqtxoplnx', '97BD544A65DB46F9A8735C93917A960F')

        self.assertRequest(req_info, request_text)


if __name__ == '__main__':
    unittest.main()




# -*- coding: utf-8 -*-

import time
import os

import oss2


# 以下代码展示了Bucket相关操作，诸如创建、删除、列举Bucket等。


# 首先初始化AccessKeyId、AccessKeySecret、Endpoint等信息。
# 通过环境变量获取，或者把诸如“<你的AccessKeyId>”替换成真实的AccessKeyId等。
access_key_id = os.getenv('OSS_TEST_ACCESS_KEY_ID', '<你的AccessKeyId>')
access_key_secret = os.getenv('OSS_TEST_ACCESS_KEY_SECRET', '<你的AccessKeySecret>')
bucket_name = os.getenv('OSS_TEST_BUCKET', '<你的Bucket>')
endpoint = os.getenv('OSS_TEST_ENDPOINT', '<你的访问域名>')


# 确认上面的参数都填写正确了
for param in (access_key_id, access_key_secret, bucket_name, endpoint):
    assert '<' not in param, '请设置参数：' + param


# 列举所有的Bucket
#   1. 先创建一个Service对象
#   2. 用oss2.BucketIterator遍历
service = oss2.Service(oss2.Auth(access_key_id, access_key_secret), endpoint)
print('\n'.join(info.name for info in oss2.BucketIterator(service)))


# 创建Bucket对象，所有Object相关的接口都可以通过Bucket对象来进行
bucket = oss2.Bucket(oss2.Auth(access_key_id, access_key_secret), endpoint, bucket_name)

# 带权限与存储类型创建bucket
bucket.create_bucket(permission=oss2.BUCKET_ACL_PRIVATE,
                     input=oss2.models.BucketCreateConfig(oss2.BUCKET_STORAGE_CLASS_STANDARD))

# 获取bucket相关信息
bucket_info = bucket.get_bucket_info()
print('name: ' + bucket_info.name)
print('storage class: ' + bucket_info.storage_class)
print('creation date: ' + bucket_info.creation_date)

# 查看Bucket的状态
bucket_stat = bucket.get_bucket_stat()
print('storage: ' + str(bucket_stat.storage_size_in_bytes))
print('object count: ' + str(bucket_stat.object_count))
print('multi part upload count: ' + str(bucket_stat.multi_part_upload_count))

# 设置bucket生命周期， 有'中文/'前缀的对象在最后修改时间之后357天失效
rule = oss2.models.LifecycleRule('lc_for_chinese_prefix', '中文/', status=oss2.models.LifecycleRule.ENABLED,
                                 expiration=oss2.models.LifecycleExpiration(days=357))

# 删除相对最后修改时间365天之后的parts
rule.abort_multipart_upload = oss2.models.AbortMultipartUpload(days=356)
# 对象最后修改时间超过180天后转为IA
rule.storage_transitions = [oss2.models.StorageTransition(days=180, storage_class=oss2.BUCKET_STORAGE_CLASS_IA)]
# 对象最后修改时间超过356天后转为ARCHIVE
rule.storage_transitions.append(oss2.models.StorageTransition(days=356, storage_class=oss2.BUCKET_STORAGE_CLASS_ARCHIVE))

lifecycle = oss2.models.BucketLifecycle([rule])
bucket.put_bucket_lifecycle(lifecycle)

# 下面只展示如何配置静态网站托管。其他的Bucket操作方式类似，可以参考tests/test_bucket.py里的内容

# 方法一：可以生成一个BucketWebsite对象来设置
bucket.put_bucket_website(oss2.models.BucketWebsite('index.html', 'error.html'))

# 方法二：可以直接设置XML
xml = '''
<WebsiteConfiguration>
    <IndexDocument>
        <Suffix>index2.html</Suffix>
    </IndexDocument>

    <ErrorDocument>
        <Key>error2.html</Key>
    </ErrorDocument>
</WebsiteConfiguration>
'''
bucket.put_bucket_website(xml)

# 方法三：可以从本地文件读取XML配置
# oss2.to_bytes()可以把unicode转换为bytes
with open('website_config.xml', 'wb') as f:
    f.write(oss2.to_bytes(xml))

with open('website_config.xml', 'rb') as f:
    bucket.put_bucket_website(f)

os.remove('website_config.xml')


# 获取配置
# 因为是分布式系统，所以配置刚刚设置好，可能还不能立即获取到，先等几秒钟
time.sleep(5)

result = bucket.get_bucket_website()
assert result.index_file == 'index2.html'
assert result.error_file == 'error2.html'


# 取消静态网站托管模式
bucket.delete_bucket_website()


# -*- coding: utf-8 -*-

import unittest
import hashlib

import oss2

from .common import *
from oss2 import to_string


class TestIterator(OssTestCase):
    def test_bucket_iterator(self):
        service = oss2.Service(oss2.Auth(OSS_ID, OSS_SECRET), OSS_ENDPOINT)
        self.assertTrue(OSS_BUCKET in (b.name for b in oss2.BucketIterator(service, max_keys=2)))

    def test_object_iterator(self):
        prefix = self.random_key('/')
        object_list = []
        dir_list = []

        # 准备文件
        for i in range(20):
            object_list.append(prefix + random_string(16))
            self.bucket.put_object(object_list[-1], random_bytes(10))

        # 准备目录
        for i in range(5):
            dir_list.append(prefix + random_string(5) + '/')
            self.bucket.put_object(dir_list[-1] + random_string(5), random_bytes(3))

        # 验证
        objects_got = []
        dirs_got = []
        for info in oss2.ObjectIterator(self.bucket, prefix, delimiter='/', max_keys=4):
            if info.is_prefix():
                dirs_got.append(info.key)
            else:
                objects_got.append(info.key)

                result = self.bucket.head_object(info.key)
                self.assertEqual(result.last_modified, info.last_modified)

        self.assertEqual(sorted(object_list), objects_got)
        self.assertEqual(sorted(dir_list), dirs_got)

        delete_keys(self.bucket, object_list)

    def test_object_iterator_chinese(self):
        for prefix in [self.random_key('中+文'), self.random_key(u'中+文')]:
            self.bucket.put_object(prefix, b'content of object')
            object_got = list(oss2.ObjectIterator(self.bucket, prefix=prefix, max_keys=1))[0].key
            self.assertEqual(to_string(prefix), object_got)

    def test_upload_iterator(self):
        prefix = self.random_key('/')
        key = prefix + random_string(16)

        upload_list = []
        dir_list = []

        # 准备分片上传
        for i in range(10):
            upload_list.append(self.bucket.init_multipart_upload(key).upload_id)

        # 准备碎片目录
        for i in range(4):
            dir_list.append(prefix + random_string(5) + '/')
            self.bucket.init_multipart_upload(dir_list[-1] + random_string(5))

        # 验证
        uploads_got = []
        dirs_got = []
        for u in oss2.MultipartUploadIterator(self.bucket, prefix=prefix, delimiter='/', max_uploads=2):
            if u.is_prefix():
                dirs_got.append(u.key)
            else:
                uploads_got.append(u.upload_id)

        self.assertEqual(sorted(upload_list), uploads_got)
        self.assertEqual(sorted(dir_list), dirs_got)

    def test_upload_iterator_chinese(self):
        upload_list = []

        p = self.random_key()
        prefix_list = [p + '中文-阿+里-巴*巴', p + u'中文-四/十*大%盗']
        for prefix in prefix_list:
            upload_list.append(self.bucket.init_multipart_upload(prefix).upload_id)

        uploads_got = []
        for prefix in prefix_list:
            listed = list(oss2.MultipartUploadIterator(self.bucket, prefix=prefix, max_uploads=1))
            uploads_got.append(listed[0].upload_id)

        self.assertEqual(sorted(upload_list), sorted(uploads_got))

    def test_object_upload_iterator(self):
        # target_object是想要列举的文件，而intact_object则不是。
        # 这里intact_object故意以target_object为前缀
        target_object = self.random_key()
        intact_object = self.random_key()

        target_list = []
        intact_list = []

        # 准备分片
        for i in range(10):
            target_list.append(self.bucket.init_multipart_upload(target_object).upload_id)
            intact_list.append(self.bucket.init_multipart_upload(intact_object).upload_id)

        # 验证：max_uploads能被分片数整除
        uploads_got = []
        for u in oss2.ObjectUploadIterator(self.bucket, target_object, max_uploads=5):
            uploads_got.append(u.upload_id)

        self.assertEqual(sorted(target_list), uploads_got)

        # 验证：max_uploads不能被分片数整除
        uploads_got = []
        for u in oss2.ObjectUploadIterator(self.bucket, target_object, max_uploads=3):
            uploads_got.append(u.upload_id)

        self.assertEqual(sorted(target_list), uploads_got)


        # 清理
        for upload_id in target_list:
            self.bucket.abort_multipart_upload(target_object, upload_id)

        for upload_id in intact_list:
            self.bucket.abort_multipart_upload(intact_object, upload_id)

    def test_part_iterator(self):
        for key in [random_string(16), '中文+_)(*&^%$#@!前缀', u'中文+_)(*&^%$#@!前缀']:
            upload_id = self.bucket.init_multipart_upload(key).upload_id

            # 准备分片
            part_list = []
            for part_number in [1, 3, 6, 7, 9, 10]:
                content = random_string(128 * 1024)
                etag = hashlib.md5(oss2.to_bytes(content)).hexdigest().upper()
                part_list.append(oss2.models.PartInfo(part_number, etag, len(content)))

                self.bucket.upload_part(key, upload_id, part_number, content)

            # 验证
            parts_got = []
            for part_info in oss2.PartIterator(self.bucket, key, upload_id):
                parts_got.append(part_info)

            self.assertEqual(len(part_list), len(parts_got))

            for i in range(len(part_list)):
                self.assertEqual(part_list[i].part_number, parts_got[i].part_number)
                self.assertEqual(part_list[i].etag, parts_got[i].etag)
                self.assertEqual(part_list[i].size, parts_got[i].size)

            self.bucket.abort_multipart_upload(key, upload_id)

    def test_live_channel_iterator(self):
        prefix = self.random_key()
        channel_name_list = []

        channel_target = oss2.models.LiveChannelInfoTarget(playlist_name = 'test.m3u8')
        channel_info = oss2.models.LiveChannelInfo(target = channel_target)
        # 准备频道
        for i in range(20):
            channel_name_list.append(prefix + random_string(16))
            self.bucket.create_live_channel(channel_name_list[-1], channel_info)

        # 验证
        live_channel_got = []
        for info in oss2.LiveChannelIterator(self.bucket, prefix, max_keys=4):
            live_channel_got.append(info.name)

            result = self.bucket.get_live_channel(info.name)
            self.assertEqual(result.description, info.description)

        self.assertEqual(sorted(channel_name_list), live_channel_got)

        for live_channel in channel_name_list:
            self.bucket.delete_live_channel(live_channel)


if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-

import base64
import hashlib
import os
import tempfile

import oss2

# 以下代码展示了上传/下载时数据校验的用法。

# OSS支持MD5、CRC64两种数据校验。MD5校验，需要用户计算出上传内容的MD5值，并放到header的`Content-MD5`中。
# CRC64校验，上传时自动完成校验，下载时需要用户校验。OSS Python SDK默认代开CRC64校验。

# 注意：断点续传上传不支持MD5，断点续传下载不支持CRC64，其它上传/下载都支持MD5和CRC64校验。

# 首先初始化AccessKeyId、AccessKeySecret、Endpoint等信息。
# 通过环境变量获取，或者把诸如“<你的AccessKeyId>”替换成真实的AccessKeyId等。
access_key_id = os.getenv('OSS_TEST_ACCESS_KEY_ID', '<你的AccessKeyId>')
access_key_secret = os.getenv('OSS_TEST_ACCESS_KEY_SECRET', '<你的AccessKeySecret>')
bucket_name = os.getenv('OSS_TEST_BUCKET', '<你的Bucket>')
endpoint = os.getenv('OSS_TEST_ENDPOINT', '<你的访问域名>')


# 确认上面的参数都填写正确了
for param in (access_key_id, access_key_secret, bucket_name, endpoint):
    assert '<' not in param, '请设置参数：' + param


def calculate_file_md5(file_name, block_size=64 * 1024):
    """计算文件的MD5
    :param file_name: 文件名
    :param block_size: 计算MD5的数据块大小，默认64KB
    :return 文件内容的MD5值
    """
    with open(file_name, 'rb') as f:
        md5 = hashlib.md5()
        while True:
            data = f.read(block_size)
            if not data:
                break
            md5.update(data)
            
    return base64.b64encode(md5.digest())

def calculate_data_md5(data):
    """计算数据的MD5
    :param data: 数据
    :return MD5值
    """
    md5 = hashlib.md5()
    md5.update(data)
    return base64.b64encode(md5.digest())


def calculate_file_crc64(file_name, block_size=64 * 1024, init_crc=0):
    """计算文件的MD5
    :param file_name: 文件名
    :param block_size: 计算MD5的数据块大小，默认64KB
    :return 文件内容的MD5值
    """
    with open(file_name, 'rb') as f:
        crc64 = oss2.utils.Crc64(init_crc)
        while True:
            data = f.read(block_size)
            if not data:
                break
            crc64.update(data)
            
    return crc64.crc

def _prepare_temp_file(content):
    """创建临时文件
    :param content: 文件内容
    :return 文件名
    """
    fd, pathname = tempfile.mkstemp(suffix='exam-progress-')
    os.write(fd, content)
    os.close(fd)
    return pathname

key = 'story.txt'
content = 'a' * 1024 * 1024

# 创建Bucket对象，所有Object相关的接口都可以通过Bucket对象来进行
bucket = oss2.Bucket(oss2.Auth(access_key_id, access_key_secret), endpoint, bucket_name)

"""
MD5校验
"""

# 上传数据
encode_md5 = calculate_data_md5(content)
bucket.put_object(key, content, headers={'Content-MD5': encode_md5})


# 上传文件
file_name = _prepare_temp_file(content)
encode_md5 = calculate_file_md5(file_name)
bucket.put_object_from_file(key, file_name, headers={'Content-MD5': encode_md5})

# 删除上传的文件
bucket.delete_object(key)


# 追加上传
# 第一次上传位置为0
encode_md5 = calculate_data_md5(content)
result = bucket.append_object(key, 0, content, headers={'Content-MD5': encode_md5})
# 第二次上传位置从result获取
bucket.append_object(key, result.next_position, content, headers={'Content-MD5': encode_md5})

# 删除上传的文件
bucket.delete_object(key)


# 分片上传
parts = []
upload_id = bucket.init_multipart_upload(key).upload_id
 
# 上传分片，每个分片单独MD5校验
encode_md5 = calculate_data_md5(content)
for i in range(3):
    result = bucket.upload_part(key, upload_id, i+1, content, headers={'Content-MD5': encode_md5})
    parts.append(oss2.models.PartInfo(i+1, result.etag, size = len(content), part_crc = result.crc))

# 完成上传并回调
result = bucket.complete_multipart_upload(key, upload_id, parts)


"""
CRC64校验
"""
# CRC校验默认打开。关闭CRC校验方法如下，oss2.Bucket(auth, endpoint, bucket_name, enable_crc=False)

# 上传数据，默认自动开启CRC校验
bucket.put_object(key, content)

# 删除上传的文件
bucket.delete_object(key)


# 追加上传，必须指定init_crc才会开启CRC校验
result = bucket.append_object(key, 0, content, init_crc=0)
# 第二次上传位置及init_crc从result获取
bucket.append_object(key, result.next_position, content, init_crc=result.crc)

# 删除上传的文件
bucket.delete_object(key)


# 分片上传，默认自动开启CRC校验
parts = []
upload_id = bucket.init_multipart_upload(key).upload_id
 
# 上传分片，每个分片单独CRC校验
for i in range(3):
    result = bucket.upload_part(key, upload_id, i+1, content)
    parts.append(oss2.models.PartInfo(i+1, result.etag, size = len(content), part_crc = result.crc))

# 完成上传并回调
result = bucket.complete_multipart_upload(key, upload_id, parts)


# 断点续传上传，默认自动开启CRC校验
pathname = _prepare_temp_file(content)
oss2.resumable_upload(bucket, key, pathname, 
                      multipart_threshold=200*1024,
                      part_size=100*1024,
                      num_threads=3)


# 下载文件
result = bucket.get_object(key)
content_got = b''
for chunk in result:
    content_got += chunk
assert result.client_crc == result.server_crc


# 下载文件到本地,默认开启CRC校验
local_file = 'download.txt'
result = bucket.get_object_to_file(key, local_file)
os.remove(local_file)
assert result.client_crc == result.server_crc


# 断点续传下载, 自动开启CRC校验，也可以用如下方法校验
oss2.resumable_download(bucket, key, local_file,
                        multiget_threshold=200*1024,
                        part_size=100*1024,
                        num_threads=3)

crc64 = calculate_file_crc64(local_file)
os.remove(local_file)

result = bucket.head_object(key)

assert str(crc64) == result.headers['x-oss-hash-crc64ecma']



# -*- coding: utf-8 -*-


import datetime
import json

from .common import *
from oss2 import to_string
from oss2.models import (BucketVersioningConfig,
                         TaggingRule,
                         Tagging,
                         LifecycleRule,
                         AbortMultipartUpload,
                         BucketLifecycle,
                         LifecycleExpiration,
                         StorageTransition,
                         NoncurrentVersionStorageTransition,
                         NoncurrentVersionExpiration)


class TestLifecycleVersioning(OssTestCase):
    def setUp(self):
        OssTestCase.setUp(self)
        self.endpoint = OSS_ENDPOINT

    def test_lifecycle_without_versioning(self):
        auth = oss2.Auth(OSS_ID, OSS_SECRET)
        bucket_name = OSS_BUCKET + "-test-lifecycle-without-versioning"
        bucket = oss2.Bucket(auth, self.endpoint, bucket_name)
        bucket.create_bucket(oss2.BUCKET_ACL_PRIVATE)
        wait_meta_sync()

        rule1 = LifecycleRule('rule1', 'tests/',
                    status=LifecycleRule.ENABLED,
                    expiration=LifecycleExpiration(days=3))

        rule2 = LifecycleRule('rule2', 'logging-',
                    status=LifecycleRule.ENABLED,
                    expiration=LifecycleExpiration(created_before_date=datetime.date(2020, 12, 12)))

        rule3 = LifecycleRule('rule3', 'tests1/',
                    status=LifecycleRule.ENABLED,
                    abort_multipart_upload=AbortMultipartUpload(days=3))

        rule4 = LifecycleRule('rule4', 'logging1-',
                    status=LifecycleRule.ENABLED,
                    abort_multipart_upload=AbortMultipartUpload(created_before_date=datetime.date(2020, 12, 12)))

        tagging_rule = TaggingRule()
        tagging_rule.add('test_key1', 'test_value1')
        tagging_rule.add('test_key2', 'test_value2')
        tagging = Tagging(tagging_rule)
        rule5 = LifecycleRule('rule5', 'logging2-',
                      status=LifecycleRule.ENABLED,
                      storage_transitions=
                      [StorageTransition(days=100, storage_class=oss2.BUCKET_STORAGE_CLASS_IA), 
                       StorageTransition(days=356, storage_class=oss2.BUCKET_STORAGE_CLASS_ARCHIVE)],
                      tagging = tagging)
        
        lifecycle = BucketLifecycle([rule1, rule2, rule3, rule4, rule5])

        bucket.put_bucket_lifecycle(lifecycle)

        lifecycle = bucket.get_bucket_lifecycle()
        self.assertEquals(5, len(lifecycle.rules))

        bucket.delete_bucket()

    def test_lifecycle_versioning(self):
        auth = oss2.Auth(OSS_ID, OSS_SECRET)
        bucket_name = OSS_BUCKET + "-test-lifecycle-versioning"
        bucket = oss2.Bucket(auth, self.endpoint, bucket_name)
        bucket.create_bucket(oss2.BUCKET_ACL_PRIVATE)
        wait_meta_sync()

        rule = LifecycleRule('rule1', 'test-prefix',
                      status=LifecycleRule.ENABLED,
                      expiration=LifecycleExpiration(expired_detete_marker=True),
                      noncurrent_version_expiration = NoncurrentVersionExpiration(30),
                      noncurrent_version_sotrage_transitions = 
                            [NoncurrentVersionStorageTransition(12, oss2.BUCKET_STORAGE_CLASS_IA),
                             NoncurrentVersionStorageTransition(20, oss2.BUCKET_STORAGE_CLASS_ARCHIVE)])

        lifecycle = BucketLifecycle([rule])

        bucket.put_bucket_lifecycle(lifecycle)

        lifecycle = bucket.get_bucket_lifecycle()
        self.assertEquals(1, len(lifecycle.rules))
        self.assertEqual('rule1', lifecycle.rules[0].id)
        self.assertEqual('test-prefix', lifecycle.rules[0].prefix)
        self.assertEquals(LifecycleRule.ENABLED, lifecycle.rules[0].status)
        self.assertEquals(True, lifecycle.rules[0].expiration.expired_detete_marker)
        self.assertEquals(30, lifecycle.rules[0].noncurrent_version_expiration.noncurrent_days)
        self.assertEquals(2, len(lifecycle.rules[0].noncurrent_version_sotrage_transitions))
        bucket.delete_bucket()
        
    def test_lifecycle_veriong_wrong(self):
        auth = oss2.Auth(OSS_ID, OSS_SECRET)
        bucket_name = OSS_BUCKET + "-test-lifecycle-versioning-wrong"
        bucket = oss2.Bucket(auth, self.endpoint, bucket_name)
        bucket.create_bucket(oss2.BUCKET_ACL_PRIVATE)
        wait_meta_sync()
        # days and expired_detete_marker cannot both exsit.
        self.assertRaises(oss2.exceptions.ClientError, LifecycleExpiration, days=10, expired_detete_marker=True)

        rule = LifecycleRule('rule1', 'test-prefix',
                      status=LifecycleRule.ENABLED,
                      expiration=LifecycleExpiration(expired_detete_marker=True),
                      noncurrent_version_expiration = NoncurrentVersionExpiration(30),
                      noncurrent_version_sotrage_transitions = 
                            [NoncurrentVersionStorageTransition(20, oss2.BUCKET_STORAGE_CLASS_IA),
                             NoncurrentVersionStorageTransition(12, oss2.BUCKET_STORAGE_CLASS_ARCHIVE)])

        lifecycle = BucketLifecycle([rule])

        # Archive transition days < IA transition days
        self.assertRaises(oss2.exceptions.InvalidArgument, bucket.put_bucket_lifecycle, lifecycle)

        bucket.delete_bucket()


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

import oss2
import time
import base64
from .common import *
from oss2.compat import to_bytes
from oss2.models import (AsyncFetchTaskConfiguration, ASYNC_FETCH_TASK_STATE_SUCCESS, 
                         ASYNC_FETCH_TASK_STATE_FETCH_SUCCESS_CALLBACK_FAILED, 
                         ASYNC_FETCH_TASK_STATE_FAILED)


class TestAsyncFetchTask(OssTestCase):
    def setUp(self):
        OssTestCase.setUp(self)
        self.endpoint = OSS_ENDPOINT

        self.fetch_object_name = 'test-async-fetch-task.txt'
        self.bucket.put_object(self.fetch_object_name, '123')

        meta = self.bucket.head_object(self.fetch_object_name)
        self.fetch_content_md5 = meta.headers.get('Content-MD5')
        self.fetch_url = self.bucket.sign_url('GET', self.fetch_object_name, 60*60)

    def test_async_fetch_task(self):
        auth = oss2.Auth(OSS_ID, OSS_SECRET)
        bucket_name = OSS_BUCKET + "-test-async-fetch-task"
        bucket = oss2.Bucket(auth, self.endpoint, bucket_name)
        bucket.create_bucket()

        object_name = self.fetch_object_name+'-destination'
        task_config = AsyncFetchTaskConfiguration(self.fetch_url, object_name=object_name, content_md5=self.fetch_content_md5, ignore_same_key=False)

        result = bucket.put_async_fetch_task(task_config)
        task_id = result.task_id
        time.sleep(5)

        result = bucket.get_async_fetch_task(task_id)
        self.assertEqual(task_id, result.task_id)
        self.assertEqual(ASYNC_FETCH_TASK_STATE_SUCCESS, result.task_state)
        self.assertEqual('', result.error_msg)
        task_config = result.task_config
        self.assertEqual(self.fetch_url, task_config.url)
        self.assertEqual(self.fetch_content_md5, task_config.content_md5)
        self.assertEqual(object_name, task_config.object_name)
        self.assertFalse(task_config.ignore_same_key)
        self.assertEqual('', task_config.host)
        self.assertEqual('', task_config.callback)

        bucket.delete_object(object_name)
        bucket.delete_bucket()

    def test_async_fetch_task_with_few_argument(self):
        auth = oss2.Auth(OSS_ID, OSS_SECRET)
        bucket_name = OSS_BUCKET + "-test-async-fetch-task"
        bucket = oss2.Bucket(auth, self.endpoint, bucket_name)
        bucket.create_bucket()

        object_name = self.fetch_object_name+'-destination'
        task_config = AsyncFetchTaskConfiguration(self.fetch_url, object_name)

        result = bucket.put_async_fetch_task(task_config)
        task_id = result.task_id
        time.sleep(5)

        result = bucket.get_async_fetch_task(task_id)
        self.assertEqual(task_id, result.task_id)
        self.assertEqual(ASYNC_FETCH_TASK_STATE_SUCCESS, result.task_state)
        self.assertEqual('', result.error_msg)
        task_config = result.task_config
        self.assertEqual(self.fetch_url, task_config.url)
        self.assertEqual('', task_config.content_md5)
        self.assertEqual(object_name, task_config.object_name)
        self.assertTrue(task_config.ignore_same_key)
        self.assertEqual('', task_config.host)
        self.assertEqual('', task_config.callback)

        bucket.delete_object(object_name)
        bucket.delete_bucket()

    def test_fetch_success_callback_failed_state(self):
        auth = oss2.Auth(OSS_ID, OSS_SECRET)
        bucket_name = OSS_BUCKET + "-test-async-fetch-task-callback"
        bucket = oss2.Bucket(auth, self.endpoint, bucket_name)
        bucket.create_bucket()

        object_name = self.fetch_object_name+'-destination'
        callback = '{"callbackUrl":"www.abc.com/callback","callbackBody":"${etag}"}'
        base64_callback = oss2.utils.b64encode_as_string(to_bytes(callback))
        task_config = AsyncFetchTaskConfiguration(self.fetch_url, object_name=object_name, callback=base64_callback)

        result = bucket.put_async_fetch_task(task_config)
        task_id = result.task_id
        time.sleep(5)

        result = bucket.get_async_fetch_task(task_id)

        self.assertEqual(task_id, result.task_id)
        self.assertEqual(ASYNC_FETCH_TASK_STATE_FETCH_SUCCESS_CALLBACK_FAILED, result.task_state)
        self.assertNotEqual('', result.error_msg)
        task_config = result.task_config
        self.assertEqual(self.fetch_url, task_config.url)
        self.assertEqual('', task_config.content_md5)
        self.assertEqual(object_name, task_config.object_name)
        self.assertTrue(task_config.ignore_same_key)
        self.assertEqual('', task_config.host)
        self.assertEqual(base64_callback, task_config.callback)

        bucket.delete_object(object_name)
        bucket.delete_bucket()


    def test_failed_state(self):
        auth = oss2.Auth(OSS_ID, OSS_SECRET)
        bucket_name = OSS_BUCKET + "-test-async-fetch-task-callback"
        bucket = oss2.Bucket(auth, self.endpoint, bucket_name)
        bucket.create_bucket()

        object_name = self.fetch_object_name+'-destination'
       
        task_config = AsyncFetchTaskConfiguration('http://invalidUrl.com', object_name=object_name)

        result = bucket.put_async_fetch_task(task_config)
        task_id = result.task_id
        time.sleep(5)

        result = bucket.get_async_fetch_task(task_id)

        self.assertEqual(task_id, result.task_id)
        self.assertEqual(ASYNC_FETCH_TASK_STATE_FAILED, result.task_state)
        self.assertNotEqual('', result.error_msg)

        bucket.delete_bucket()

    def test_ignore_same_key(self):
        auth = oss2.Auth(OSS_ID, OSS_SECRET)
        bucket_name = OSS_BUCKET + "-test-async-fetch-task"
        bucket = oss2.Bucket(auth, self.endpoint, bucket_name)
        bucket.create_bucket()

        object_name = self.fetch_object_name+'-destination'
        bucket.put_object(object_name, 'test-content')

        task_config = AsyncFetchTaskConfiguration(self.fetch_url, object_name=object_name, ignore_same_key=False)
        result = bucket.put_async_fetch_task(task_config)
        task_id = result.task_id
        self.assertNotEqual('', task_id)

        task_config = AsyncFetchTaskConfiguration(self.fetch_url, object_name=object_name, ignore_same_key=True)
        self.assertRaises(oss2.exceptions.ObjectAlreadyExists, bucket.put_async_fetch_task, task_config)

        task_config = AsyncFetchTaskConfiguration(self.fetch_url, object_name=object_name)
        self.assertRaises(oss2.exceptions.ObjectAlreadyExists, bucket.put_async_fetch_task, task_config)

        bucket.delete_object(object_name)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

from .common import *
from oss2.headers import *


class TestHeaders(OssTestCase):
    def test_check_requestHeader(self):
        myHeader = RequestHeader()
        
        myHeader.set_server_side_encryption(algorithm="AES256")
        self.assertEqual(myHeader["x-oss-server-side-encryption"], "AES256")

        myHeader.set_server_side_encryption(algorithm='KMS')
        self.assertEqual(myHeader["x-oss-server-side-encryption"], "KMS")
        self.assertTrue("x-oss-server-side-encryption-key-id" not in myHeader)

        myHeader.set_server_side_encryption(algorithm="KMS", cmk_id="1111")
        self.assertEqual(myHeader["x-oss-server-side-encryption"], "KMS")
        self.assertEqual(myHeader["x-oss-server-side-encryption-key-id"], "1111")

        myHeader.set_server_side_encryption(algorithm="aaa")
        self.assertTrue("x-oss-server-side-encryption" not in myHeader)
        self.assertTrue("x-oss-server-side-encryption-key-id" not in myHeader)


if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-

import time
import datetime
import json
import base64
import hmac
import hashlib
import os
import crcmod
import requests


# 以下代码展示了PostObject的用法。PostObject不依赖于OSS Python SDK。

# POST表单域的详细说明请参RFC2388 https://tools.ietf.org/html/rfc2388
# PostObject的官网 https://help.aliyun.com/document_detail/31988.html
# PostObject错误及排查 https://yq.aliyun.com/articles/58524

# 首先初始化AccessKeyId、AccessKeySecret、Endpoint等信息。
# 通过环境变量获取，或者把诸如“<你的AccessKeyId>”替换成真实的AccessKeyId等。
access_key_id = os.getenv('OSS_TEST_ACCESS_KEY_ID', '<你的AccessKeyId>')
access_key_secret = os.getenv('OSS_TEST_ACCESS_KEY_SECRET', '<你的AccessKeySecret>')
bucket_name = os.getenv('OSS_TEST_BUCKET', '<你的Bucket>')
endpoint = os.getenv('OSS_TEST_ENDPOINT', '<你的访问域名>')

# 确认上面的参数都填写正确了
for param in (access_key_id, access_key_secret, bucket_name, endpoint):
    assert '<' not in param, '请设置参数：' + param

def calculate_crc64(data):
    """计算文件的MD5
    :param data: 数据
    :return 数据的MD5值
    """
    _POLY = 0x142F0E1EBA9EA3693
    _XOROUT = 0XFFFFFFFFFFFFFFFF
    
    crc64 = crcmod.Crc(_POLY, initCrc=0, xorOut=_XOROUT)
    crc64.update(data)

    return crc64.crcValue

def build_gmt_expired_time(expire_time):
    """生成GMT格式的请求超时时间
    :param int expire_time: 超时时间，单位秒
    :return str GMT格式的超时时间
    """
    now = int(time.time())
    expire_syncpoint  = now + expire_time
    
    expire_gmt = datetime.datetime.fromtimestamp(expire_syncpoint).isoformat()
    expire_gmt += 'Z'
    
    return expire_gmt

def build_encode_policy(expired_time, condition_list):
    """生成policy
    :param int expired_time: 超时时间，单位秒
    :param list condition_list: 限制条件列表
    """ 
    policy_dict = {}
    policy_dict['expiration'] = build_gmt_expired_time(expired_time)
    policy_dict['conditions'] = condition_list
    
    policy = json.dumps(policy_dict).strip()
    policy_encode = base64.b64encode(policy)
    
    return policy_encode

def build_signature(access_key_secret, encode_policy):
    """生成签名
    :param str access_key_secret: access key secret
    :param str encode_policy: 编码后的Policy
    :return str 请求签名
    """
    h = hmac.new(access_key_secret, encode_policy, hashlib.sha1)
    signature = base64.encodestring(h.digest()).strip()
    return signature

def bulid_callback(cb_url, cb_body, cb_body_type=None, cb_host=None):
    """生成callback字符串
    :param str cb_url: 回调服务器地址，文件上传成功后OSS向此url发送回调请求
    :param str cb_body: 发起回调请求的Content-Type，默认application/x-www-form-urlencoded
    :param str cb_body_type: 发起回调时请求body
    :param str cb_host: 发起回调请求时Host头的值
    :return str 编码后的Callback
    """
    callback_dict = {}
    
    callback_dict['callbackUrl'] = cb_url
    
    callback_dict['callbackBody'] = cb_body
    if cb_body_type is None:
        callback_dict['callbackBodyType'] = 'application/x-www-form-urlencoded'
    else:
        callback_dict['callbackBodyType'] = cb_body_type
    
    if cb_host is not None:
        callback_dict['callbackHost'] = cb_host
        
    callback_param = json.dumps(callback_dict).strip()
    base64_callback = base64.b64encode(callback_param);
    
    return base64_callback

def build_post_url(endpoint, bucket_name):
    """生成POST请求URL
    :param str endpoint: endpoint
    :param str bucket_name: bucket name
    :return str POST请求URL
    """
    if endpoint.startswith('http://'):
        return endpoint.replace('http://', 'http://{0}.'.format(bucket_name))
    elif endpoint.startswith('https://'):
        return endpoint.replace('https://', 'https://{0}.'.format(bucket_name))
    else:
        return 'http://{0}.{1}'.format(bucket_name, endpoint)

def build_post_body(field_dict, boundary):
    """生成POST请求Body
    :param dict field_dict: POST请求表单域
    :param str boundary: 表单域的边界字符串
    :return str POST请求Body
    """
    post_body = b''

    # 编码表单域
    for k,v in field_dict.iteritems():
        if k != 'content' and k != 'content-type':
            post_body += '''--{0}\r\nContent-Disposition: form-data; name=\"{1}\"\r\n\r\n{2}\r\n'''.format(boundary, k, v)
    
    # 上传文件的内容，必须作为最后一个表单域
    post_body += '''--{0}\r\nContent-Disposition: form-data; name=\"file\"; filename=\"{1}\"\r\nContent-Type: {2}\r\n\r\n{3}'''.format(
    boundary, field_dict['key'], field_dict['content-type'], field_dict['content'])
    
    # 加上表单域结束符
    post_body += '\r\n--{0}--\r\n'.format(boundary)

    return post_body

def build_post_headers(body_len, boundary, headers=None):
    """生气POST请求Header
    :param str body_len: POST请求Body长度
    :param str boundary: 表单域的边界字符串
    :param dict 请求Header
    """
    headers = headers if headers else {}
    headers['Content-Length'] = str(body_len)
    headers['Content-Type'] = 'multipart/form-data; boundary={0}'.format(boundary)

    return headers


# POST请求表单域，注意大小写
field_dict = {}
# object名称
field_dict['key'] = 'post.txt'
# access key id
field_dict['OSSAccessKeyId'] = access_key_id
# Policy包括超时时间(单位秒)和限制条件condition
field_dict['policy'] = build_encode_policy(120, [['eq','$bucket', bucket_name],
                                                 ['content-length-range', 0, 104857600]])
# 请求签名
field_dict['Signature'] = build_signature(access_key_secret, field_dict['policy']) 
# 临时用户Token，当使用临时用户密钥时Token必填；非临时用户填空或不填
field_dict['x-oss-security-token'] = ''
# Content-Disposition
field_dict['Content-Disposition'] = 'attachment;filename=download.txt'
# 用户自定义meta
field_dict['x-oss-meta-uuid'] = 'uuid-xxx'
# callback，没有回调需求不填该域
field_dict['callback'] = bulid_callback('http://oss-demo.aliyuncs.com:23450',
                                        'filename=${object}&size=${size}&mimeType=${mimeType}',
                                        'application/x-www-form-urlencoded') 
# callback中的自定义变量，没有回调不填该域
field_dict['x:var1'] = 'callback-var1-val'
# 上传文件内容
field_dict['content'] = 'a'*64
# 上传文件类型
field_dict['content-type'] = 'text/plain'

# 表单域的边界字符串，一般为随机字符串
boundary = '9431149156168'

# 发送POST请求
body = build_post_body(field_dict, boundary)
headers = build_post_headers(len(body), boundary)

resp = requests.post(build_post_url(endpoint, bucket_name),
                     data=body,
                     headers=headers)

# 确认请求结果
assert resp.status_code == 200
assert resp.content == '{"Status":"OK"}'
assert resp.headers['x-oss-hash-crc64ecma'] == str(calculate_crc64(field_dict['content']))



__version__ = '2.15.0'

from . import models, exceptions, defaults

from .api import Service, Bucket
from .auth import Auth, AuthV2, AnonymousAuth, StsAuth, AUTH_VERSION_1, AUTH_VERSION_2, make_auth, ProviderAuth, ProviderAuthV2
from .http import Session, CaseInsensitiveDict
from .credentials import EcsRamRoleCredentialsProvider, EcsRamRoleCredential, CredentialsProvider, StaticCredentialsProvider

from .iterators import (BucketIterator, ObjectIterator, ObjectIteratorV2,
                        MultipartUploadIterator, ObjectUploadIterator,
                        PartIterator, LiveChannelIterator)


from .resumable import resumable_upload, resumable_download, ResumableStore, ResumableDownloadStore, determine_part_size
from .resumable import make_upload_store, make_download_store


from .compat import to_bytes, to_string, to_unicode, urlparse, urlquote, urlunquote

from .utils import SizedFileAdapter, make_progress_adapter
from .utils import content_type_by_name, is_valid_bucket_name, is_valid_endpoint
from .utils import http_date, http_to_unixtime, iso8601_to_unixtime, date_to_iso8601, iso8601_to_date


from .models import BUCKET_ACL_PRIVATE, BUCKET_ACL_PUBLIC_READ, BUCKET_ACL_PUBLIC_READ_WRITE
from .models import SERVER_SIDE_ENCRYPTION_AES256, SERVER_SIDE_ENCRYPTION_KMS, SERVER_SIDE_ENCRYPTION_SM4, KMS_DATA_ENCRYPTION_SM4
from .models import OBJECT_ACL_DEFAULT, OBJECT_ACL_PRIVATE, OBJECT_ACL_PUBLIC_READ, OBJECT_ACL_PUBLIC_READ_WRITE
from .models import BUCKET_STORAGE_CLASS_STANDARD, BUCKET_STORAGE_CLASS_IA, BUCKET_STORAGE_CLASS_ARCHIVE, BUCKET_STORAGE_CLASS_COLD_ARCHIVE
from .models import BUCKET_VERSIONING_ENABLE, BUCKET_VERSIONING_SUSPEND 
from .models import BUCKET_DATA_REDUNDANCY_TYPE_LRS, BUCKET_DATA_REDUNDANCY_TYPE_ZRS

from .crypto import LocalRsaProvider, AliKMSProvider, RsaProvider,EncryptionMaterials
from .crypto_bucket import CryptoBucket
import logging

logger = logging.getLogger('oss2')


def set_file_logger(file_path, name="oss2", level=logging.INFO, format_string=None):
    global logger
    if not format_string:
        format_string = "%(asctime)s %(name)s [%(levelname)s] %(thread)d : %(message)s"
    logger = logging.getLogger(name)
    logger.setLevel(level)
    fh = logging.FileHandler(file_path)
    fh.setLevel(level)
    formatter = logging.Formatter(format_string)
    fh.setFormatter(formatter)
    logger.addHandler(fh)


def set_stream_logger(name='oss2', level=logging.DEBUG, format_string=None):
    global logger
    if not format_string:
        format_string = "%(asctime)s %(name)s [%(levelname)s] %(thread)d : %(message)s"
    logger = logging.getLogger(name)
    logger.setLevel(level)
    fh = logging.StreamHandler()
    fh.setLevel(level)
    formatter = logging.Formatter(format_string)
    fh.setFormatter(formatter)
    logger.addHandler(fh)
